#!/usr/bin/env python3
import os, subprocess, argparse, time, transitions, base64
from pathlib import Path
import _workerimpl, _zmqssh

SCRIPTDIR=Path(os.path.dirname(os.path.realpath(__file__)))

################################################################################

parser = argparse.ArgumentParser(description='orkbb-worker')
parser.add_argument('--project', metavar="projectfile", required=True, help='path of json project file' )
parser.add_argument('--outputdir', default=SCRIPTDIR/"output", metavar="outputdir", help='outputdirectory' )
parser.add_argument("--debug",action="store_true")
parser.add_argument("--verbose",action="store_true")
args = vars(parser.parse_args())

################################################################################

VERBOSE=args["verbose"]
OUTPUTDIR = Path(args["outputdir"]).absolute()
BASEDIR=OUTPUTDIR/"builds"
print("remove old outputdir<%s>"%OUTPUTDIR)
os.system("rm -rf %s"%OUTPUTDIR)
os.system("mkdir -p %s"%OUTPUTDIR)

################################################################################

config = _workerimpl.Config(Path(args["project"]),OUTPUTDIR)
print(config)

#########################################
def dirsubst(inpstr,REPODIR):
  outstr = inpstr.replace("$REPODIR",str(REPODIR))
  outstr = outstr.replace("$STAGE",str(STAGE))
  return outstr
################################################################################
def tunnelledZMQ():
  global config
  #impl = _zmqssh.ZmqSsh2
  impl = _zmqssh.Direct

  rval = impl(ipaddr=(config.master_ssh_host,config.master_ssh_port),
              user=config.master_ssh_user,
              key="~/.ssh/id_rsa",
              lbindaddr=(config.master_zmq_host, config.master_zmq_port) )
  return rval
#########################################

class Worker(object):
  states = ["booting","connecting","waiting","initializing","fetching","building","reporting"]

  def __init__(self):
    ####################################
    self.name = "worker"
    self.cur_sha = None
    self.cur_outdir = None
    self.buildtime = 0
    self.returncode = 0
    ####################################
    self._fsm = transitions.Machine(model=self,queued=True,states=Worker.states,initial="booting")
    self._fsm.add_transition(trigger='boot', source='booting', dest='connecting')
    self._fsm.add_transition(trigger='advance', source='connecting', dest='waiting')
    self._fsm.add_transition(trigger='advance', source='waiting', dest='initializing')
    self._fsm.add_transition(trigger='advance', source='initializing', dest='fetching')
    self._fsm.add_transition(trigger='advance', source='fetching', dest='building')
    self._fsm.add_transition(trigger='advance', source='building', dest='reporting')
    self._fsm.add_transition(trigger='advance', source='reporting', dest='waiting')
    self._fsm.add_transition('reset', '*', 'waiting')
    ####################################
    self._fsm.on_enter_connecting('beginConnecting')
    self._fsm.on_enter_waiting('beginWaiting')
    self._fsm.on_enter_initializing('beginInitializing')
    self._fsm.on_enter_fetching('beginFetching')
    self._fsm.on_enter_building('beginBuilding')
    self._fsm.on_enter_reporting('beginReporting')
    ####################################

  ####################################
  def sendMessage(self,message):
    reply = None
    with tunnelledZMQ() as socket:
      socket.send_json(message)
      keepwaiting = True
      while keepwaiting:
        qres = socket.recv_json(timeout=5)
        if qres==None:
          print("worker-buildstatus timeout, waiting...")
        else:
          keepwaiting = False
          reply = qres
    return reply
  ####################################
  def beginConnecting(self):
    print("CONNECTING")
    keepconnecting=True
    while keepconnecting:
      reply = self.sendMessage({
        "command": "worker-connect",
        "worker": config.name
      })
      keepconnecting=(reply==None)
  ####################################
  def beginWaiting(self):
    keepwaiting=True
    while keepwaiting:
      reply=self.sendMessage({
        "command": "worker-waiting",
        "worker": config.name
      })
      if reply!=None:
        print("WAIT reply<%s>"%reply)
        assert(reply["state"] == "wait")
        if "jobindex" in reply:
          self.jobindex = reply["jobindex"]
          keepwaiting = False
  ####################################
  def beginInitializing(self):
    message = {"command": "worker-initializing",
               "worker": config.name}
    reply = self.sendMessage(message)
    if reply["state"] != "init":
      self.reset()
      return
    print("INIT<%s>"%reply)
  ####################################
  def beginFetching(self):
    message = {"command": "worker-fetching",
               "worker": config.name}
    reply = self.sendMessage(message)
    print("BEGINFETCH"%reply)
    if reply["state"] != "fetch":
      self.reset()
      return
    variant = reply["variant"]
    os.chdir(SCRIPTDIR)
    self.cur_sha = reply["sha"]
    self.cur_outdir = OUTPUTDIR/variant
    cmdlist = [ "./worker_fetch_branch.py",
                "--sha", self.cur_sha,
                "--gitURL", reply["repourl"],
                "--outputdir", self.cur_outdir,
    ]
    if reply["skipLFS"]:
      cmdlist += ["--skipLFS"]
    proc = subprocess.Popen(cmdlist)
    proc.wait()
    print("ENDFETCH")
  ####################################
  def beginBuilding(self):
    message = {"command": "worker-building",
               "worker": config.name}
    reply = self.sendMessage(message)
    print("BEGINBUILD<%s>"%reply)
    if reply["state"] != "build":
      self.reset()
      return
    os.chdir(SCRIPTDIR)

    print(reply["commandlist"])

    cmdlist = [ "./worker_build_branch.py",
                "--sha", self.cur_sha,
                "--outputdir", self.cur_outdir,
                "--workingdir", reply["workingdir"],
    ]

    for item in reply["commandlist"]:
      cmdlist += ["--command",item]

    print(cmdlist)

    if VERBOSE:
      cmdlist += ["--verbose"]

    time_start = time.time()
    proc = subprocess.Popen(cmdlist)
    proc.wait()
    print("ENDBUILD ret<%d>"%proc.returncode)
    self.returncode = proc.returncode
    time_end = time.time()
    time_diff = time_end-time_start
    self.buildtime = time.strftime('%Hh:%Mm:%Ss', time.gmtime(time_diff))
  ####################################
  def beginReporting(self):
    print("REPORT")
    build_dir = self.cur_outdir/"builds"/self.cur_sha
    cmdlist = [ "./process_log.py",
                "--basedir", self.cur_outdir,
                "--directory", build_dir
              ]
    proc = subprocess.Popen(cmdlist)
    proc.wait()
    log_html_path = build_dir/"stdout.html"
    f = open(log_html_path,'rb')
    bytes = bytearray(f.read())
    log_html_enc = base64.b64encode(bytes).decode("utf-8")
    f.close()
    message = {"command": "worker-reporting",
               "worker": config.name,
               "buildtime": self.buildtime,
               "log_html_enc": log_html_enc }
    if self.returncode==0:
      message["status"] = "OK"
    else:
      message["status"] = "ERROR"
    reply = self.sendMessage(message)
    os.system("rm -rf %s"%self.cur_outdir)
    if reply["state"] != "report":
      self.reset()
      return
    time.sleep(config.sleeptime)
  ####################################

################################################################################

worker = Worker()
worker.boot()

################################################################################
# Main Loop
################################################################################

while True:
  worker.advance()



#!/usr/bin/env python3

from obt.deco import Deco
from obt import command

def main():
    deco = Deco()

    print(deco.inf("Disabling systemd user instance from starting on boot (unlinger)"))
    print(deco.inf("User services will only run when logged in"))

    # Disable linger for the current user
    exit_code = command.run(["loginctl", "disable-linger"])

    if exit_code == 0:
        print(deco.key("User systemd instance disabled from boot"))
    else:
        print(deco.err("Failed to disable user systemd instance"))
        exit(1)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
Unit tests for cpp_entities_v2.py data models
"""
import unittest
from obt.cpp_entities_v2 import (
    Entity, Location, Member, Parameter,
    EntityType, LocationType, AccessLevel, MemberType
)

class TestLocation(unittest.TestCase):
    """Test Location data model"""
    
    def test_location_creation(self):
        """Test creating a location"""
        loc = Location(
            file_path="/path/to/file.cpp",
            line_number=42,
            column_number=10,
            location_type=LocationType.DEFINITION,
            has_body=True
        )
        
        self.assertEqual(loc.file_path, "/path/to/file.cpp")
        self.assertEqual(loc.line_number, 42)
        self.assertEqual(loc.column_number, 10)
        self.assertEqual(loc.location_type, LocationType.DEFINITION)
        self.assertTrue(loc.has_body)
    
    def test_location_equality(self):
        """Test location equality based on file and line"""
        loc1 = Location("/path/file.cpp", 42, column_number=10)
        loc2 = Location("/path/file.cpp", 42, column_number=20)  # Different column
        loc3 = Location("/path/file.cpp", 43)  # Different line
        
        self.assertEqual(loc1, loc2)  # Same file and line
        self.assertNotEqual(loc1, loc3)  # Different line
    
    def test_location_hash(self):
        """Test location hashing for use in sets"""
        loc1 = Location("/path/file.cpp", 42)
        loc2 = Location("/path/file.cpp", 42)
        loc3 = Location("/path/file.cpp", 43)
        
        locations = {loc1, loc2, loc3}
        self.assertEqual(len(locations), 2)  # loc1 and loc2 are same

class TestMember(unittest.TestCase):
    """Test Member data model"""
    
    def test_member_creation(self):
        """Test creating a member"""
        member = Member(
            name="m_data",
            member_type=MemberType.FIELD,
            data_type="int",
            access_level=AccessLevel.PRIVATE,
            is_static=False,
            line_number=10
        )
        
        self.assertEqual(member.name, "m_data")
        self.assertEqual(member.member_type, MemberType.FIELD)
        self.assertEqual(member.data_type, "int")
        self.assertEqual(member.access_level, AccessLevel.PRIVATE)
        self.assertFalse(member.is_static)
    
    def test_member_equality(self):
        """Test member equality based on name and type"""
        m1 = Member("foo", MemberType.METHOD)
        m2 = Member("foo", MemberType.METHOD)  # Same
        m3 = Member("foo", MemberType.FIELD)   # Different type
        m4 = Member("bar", MemberType.METHOD)  # Different name
        
        self.assertEqual(m1, m2)
        self.assertNotEqual(m1, m3)
        self.assertNotEqual(m1, m4)
    
    def test_method_member(self):
        """Test creating a method member"""
        method = Member(
            name="process",
            member_type=MemberType.METHOD,
            data_type="void",
            access_level=AccessLevel.PUBLIC,
            is_virtual=True,
            is_const=True,
            signature="void process() const"
        )
        
        self.assertEqual(method.member_type, MemberType.METHOD)
        self.assertTrue(method.is_virtual)
        self.assertTrue(method.is_const)
        self.assertEqual(method.signature, "void process() const")

class TestEntity(unittest.TestCase):
    """Test Entity data model"""
    
    def test_entity_creation(self):
        """Test creating an entity"""
        entity = Entity(
            canonical_name="ork::util::Context",
            short_name="Context",
            entity_type=EntityType.CLASS,
            namespace="ork::util"
        )
        
        self.assertEqual(entity.canonical_name, "ork::util::Context")
        self.assertEqual(entity.short_name, "Context")
        self.assertEqual(entity.entity_type, EntityType.CLASS)
        self.assertEqual(entity.namespace, "ork::util")
        self.assertFalse(entity.is_template)
        self.assertEqual(len(entity.locations), 0)
        self.assertEqual(len(entity.members), 0)
    
    def test_template_entity(self):
        """Test creating a template entity"""
        entity = Entity(
            canonical_name="std::vector",
            short_name="vector",
            entity_type=EntityType.CLASS,
            namespace="std",
            is_template=True,
            template_params="<typename T, typename Allocator>"
        )
        
        self.assertTrue(entity.is_template)
        self.assertEqual(entity.template_params, "<typename T, typename Allocator>")
    
    def test_add_location(self):
        """Test adding locations to entity"""
        entity = Entity("MyClass", "MyClass", EntityType.CLASS)
        
        # Add declaration
        loc1 = Location("/path/file.h", 10, location_type=LocationType.DECLARATION)
        self.assertTrue(entity.add_location(loc1))
        self.assertEqual(len(entity.locations), 1)
        
        # Add definition
        loc2 = Location("/path/file.cpp", 50, location_type=LocationType.DEFINITION)
        self.assertTrue(entity.add_location(loc2))
        self.assertEqual(len(entity.locations), 2)
        
        # Try to add duplicate location
        loc3 = Location("/path/file.h", 10, location_type=LocationType.DECLARATION)
        self.assertFalse(entity.add_location(loc3))
        self.assertEqual(len(entity.locations), 2)
        
        # Update existing location with definition
        loc4 = Location("/path/file.h", 10, location_type=LocationType.DEFINITION)
        self.assertTrue(entity.add_location(loc4))
        self.assertEqual(len(entity.locations), 2)
        # Check it was updated
        self.assertEqual(entity.locations[0].location_type, LocationType.DEFINITION)
    
    def test_get_primary_location(self):
        """Test getting primary location (prefer definition)"""
        entity = Entity("MyClass", "MyClass", EntityType.CLASS)
        
        # Add declaration first
        decl = Location("/path/file.h", 10, location_type=LocationType.DECLARATION)
        entity.add_location(decl)
        
        # Primary should be declaration (only one available)
        self.assertEqual(entity.get_primary_location(), decl)
        
        # Add definition
        defn = Location("/path/file.cpp", 50, location_type=LocationType.DEFINITION)
        entity.add_location(defn)
        
        # Primary should now be definition
        self.assertEqual(entity.get_primary_location(), defn)
    
    def test_add_member(self):
        """Test adding members to entity"""
        entity = Entity("MyClass", "MyClass", EntityType.CLASS)
        
        # Add field
        field = Member("m_data", MemberType.FIELD, data_type="int")
        self.assertTrue(entity.add_member(field))
        self.assertEqual(len(entity.members), 1)
        
        # Add method
        method = Member("process", MemberType.METHOD, data_type="void")
        self.assertTrue(entity.add_member(method))
        self.assertEqual(len(entity.members), 2)
        
        # Try to add duplicate
        dup_field = Member("m_data", MemberType.FIELD, data_type="double")
        self.assertFalse(entity.add_member(dup_field))
        self.assertEqual(len(entity.members), 2)
    
    def test_get_members_by_access(self):
        """Test filtering members by access level"""
        entity = Entity("MyClass", "MyClass", EntityType.CLASS)
        
        # Add members with different access levels
        pub_method = Member("publicMethod", MemberType.METHOD, 
                           access_level=AccessLevel.PUBLIC)
        priv_field = Member("m_data", MemberType.FIELD,
                           access_level=AccessLevel.PRIVATE)
        prot_method = Member("protectedMethod", MemberType.METHOD,
                            access_level=AccessLevel.PROTECTED)
        
        entity.add_member(pub_method)
        entity.add_member(priv_field)
        entity.add_member(prot_method)
        
        # Test filtering
        public_members = entity.get_members_by_access(AccessLevel.PUBLIC)
        self.assertEqual(len(public_members), 1)
        self.assertEqual(public_members[0].name, "publicMethod")
        
        private_members = entity.get_members_by_access(AccessLevel.PRIVATE)
        self.assertEqual(len(private_members), 1)
        self.assertEqual(private_members[0].name, "m_data")
    
    def test_get_members_by_type(self):
        """Test filtering members by type"""
        entity = Entity("MyClass", "MyClass", EntityType.CLASS)
        
        # Add different member types
        field1 = Member("m_data", MemberType.FIELD)
        field2 = Member("m_count", MemberType.FIELD)
        method = Member("process", MemberType.METHOD)
        
        entity.add_member(field1)
        entity.add_member(field2)
        entity.add_member(method)
        
        # Test filtering
        fields = entity.get_members_by_type(MemberType.FIELD)
        self.assertEqual(len(fields), 2)
        
        methods = entity.get_members_by_type(MemberType.METHOD)
        self.assertEqual(len(methods), 1)
        self.assertEqual(methods[0].name, "process")
    
    def test_is_derived(self):
        """Test checking if class is derived"""
        base_class = Entity("Base", "Base", EntityType.CLASS)
        self.assertFalse(base_class.is_derived())
        
        derived_class = Entity("Derived", "Derived", EntityType.CLASS)
        derived_class.base_classes = ["Base"]
        self.assertTrue(derived_class.is_derived())
    
    def test_merge_entities(self):
        """Test merging two entities"""
        # Create entity with partial info
        entity1 = Entity("MyClass", "MyClass", EntityType.CLASS)
        entity1.add_location(Location("/path/file.h", 10))
        entity1.add_member(Member("method1", MemberType.METHOD))
        
        # Create another entity with more info
        entity2 = Entity("MyClass", "MyClass", EntityType.CLASS)
        entity2.add_location(Location("/path/file.cpp", 50))
        entity2.add_member(Member("method2", MemberType.METHOD))
        entity2.return_type = "void"
        entity2.is_virtual = True
        
        # Merge
        self.assertTrue(entity1.merge_with(entity2))
        
        # Check merged data
        self.assertEqual(len(entity1.locations), 2)
        self.assertEqual(len(entity1.members), 2)
        self.assertEqual(entity1.return_type, "void")
        self.assertTrue(entity1.is_virtual)
        
        # Can't merge different entities
        entity3 = Entity("OtherClass", "OtherClass", EntityType.CLASS)
        self.assertFalse(entity1.merge_with(entity3))
    
    def test_function_entity(self):
        """Test creating a function entity"""
        entity = Entity(
            canonical_name="process",
            short_name="process",
            entity_type=EntityType.FUNCTION,
            return_type="int",
            is_const=True,
            is_noexcept=True
        )
        
        # Add parameters
        param1 = Parameter("input", "const std::string&", is_const=True, is_reference=True)
        param2 = Parameter("flags", "int", default_value="0")
        entity.parameters = [param1, param2]
        
        self.assertEqual(entity.entity_type, EntityType.FUNCTION)
        self.assertEqual(entity.return_type, "int")
        self.assertTrue(entity.is_const)
        self.assertTrue(entity.is_noexcept)
        self.assertEqual(len(entity.parameters), 2)
        self.assertEqual(entity.parameters[0].name, "input")
        self.assertEqual(entity.parameters[1].default_value, "0")
    
    def test_enum_entity(self):
        """Test creating an enum entity"""
        entity = Entity(
            canonical_name="Color",
            short_name="Color",
            entity_type=EntityType.ENUM,
            is_enum_class=True,
            underlying_type="uint32_t"
        )
        
        # Add enum values
        red = Member("RED", MemberType.ENUM_VALUE, value="0xFF0000")
        green = Member("GREEN", MemberType.ENUM_VALUE, value="0x00FF00")
        blue = Member("BLUE", MemberType.ENUM_VALUE, value="0x0000FF")
        
        entity.add_member(red)
        entity.add_member(green)
        entity.add_member(blue)
        
        self.assertEqual(entity.entity_type, EntityType.ENUM)
        self.assertTrue(entity.is_enum_class)
        self.assertEqual(entity.underlying_type, "uint32_t")
        self.assertEqual(len(entity.members), 3)
        
        # Check enum values
        enum_values = entity.get_members_by_type(MemberType.ENUM_VALUE)
        self.assertEqual(len(enum_values), 3)
        self.assertEqual(enum_values[0].value, "0xFF0000")
    
    def test_typedef_entity(self):
        """Test creating a typedef/alias entity"""
        entity = Entity(
            canonical_name="StringList",
            short_name="StringList",
            entity_type=EntityType.TYPEDEF,
            aliased_type="std::vector<std::string>",
            is_using_alias=True
        )
        
        self.assertEqual(entity.entity_type, EntityType.TYPEDEF)
        self.assertEqual(entity.aliased_type, "std::vector<std::string>")
        self.assertTrue(entity.is_using_alias)

class TestParameter(unittest.TestCase):
    """Test Parameter data model"""
    
    def test_parameter_creation(self):
        """Test creating function parameters"""
        # Simple parameter
        param1 = Parameter("count", "int")
        self.assertEqual(param1.name, "count")
        self.assertEqual(param1.param_type, "int")
        self.assertFalse(param1.is_const)
        self.assertFalse(param1.is_reference)
        
        # Const reference parameter
        param2 = Parameter(
            name="input",
            param_type="std::string",
            is_const=True,
            is_reference=True
        )
        self.assertTrue(param2.is_const)
        self.assertTrue(param2.is_reference)
        
        # Parameter with default value
        param3 = Parameter(
            name="flags",
            param_type="int",
            default_value="0"
        )
        self.assertEqual(param3.default_value, "0")
        
        # Unnamed parameter
        param4 = Parameter(None, "void*", is_pointer=True)
        self.assertIsNone(param4.name)
        self.assertTrue(param4.is_pointer)
        
        # Rvalue reference
        param5 = Parameter(
            name="data",
            param_type="std::vector<int>",
            is_rvalue_ref=True
        )
        self.assertTrue(param5.is_rvalue_ref)

if __name__ == "__main__":
    unittest.main()


#!/usr/bin/env python3
###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import argparse

def parse_search_args(description, require_dep=False):
    """
    Common argument parser for search commands.
    Returns parsed args and processed options.
    """
    parser = argparse.ArgumentParser(description=description)
    
    # Add dep argument if required
    if require_dep:
        parser.add_argument('--dep', help='dep to search', required=True)
    
    # File filtering options
    parser.add_argument('--filetypes', 
                       help='File extensions to search (e.g., ".cpp:.h:.py")',
                       type=str,
                       default=None)
    
    # Search mode options
    parser.add_argument('-i', '--case-insensitive',
                       help='Case-insensitive search (useful for finding API usage regardless of naming)',
                       action='store_true')
    
    parser.add_argument('-l', '--files-only',
                       help='List only filenames with matches, not the matching lines (faster for overview)',
                       action='store_true')
    
    parser.add_argument('-r', '--regex',
                       help='Treat pattern as regular expression instead of literal string (can specify multiple for AND logic)',
                       action='append',
                       dest='regex_patterns')
    
    parser.add_argument('-w', '--whole-word',
                       help='Match whole words only (avoid partial matches like "get" in "getAsset")',
                       action='store_true')
    
    parser.add_argument('-v', '--invert-match',
                       help='Show lines that do NOT match the pattern (useful for filtering)',
                       action='store_true')
    
    # Context options
    parser.add_argument('-C', '--context',
                       help='Show N lines of context around matches (highlighted with delimiters)',
                       type=int,
                       default=0,
                       metavar='N')
    
    parser.add_argument('-B', '--before-context',
                       help='Show N lines before each match',
                       type=int,
                       default=0,
                       metavar='N')
    
    parser.add_argument('-A', '--after-context', 
                       help='Show N lines after each match',
                       type=int,
                       default=0,
                       metavar='N')
    
    # Required positional argument (unless -r is used)
    parser.add_argument('keywords', 
                       metavar='KEYWORD', 
                       type=str, 
                       nargs='*',  # Changed to * to make optional when -r is used
                       help='search keywords or patterns')
    
    args = parser.parse_args()
    
    # Process filetypes into ext_set
    ext_set = None
    if args.filetypes:
        # Split by colon and ensure each has a dot
        extensions = args.filetypes.split(':')
        ext_set = set()
        for ext in extensions:
            # Add dot if not present
            if not ext.startswith('.'):
                ext = '.' + ext
            ext_set.add(ext)
    
    # Process context options
    # If -C is specified, it overrides -A and -B
    if args.context > 0:
        args.before_context = args.context
        args.after_context = args.context
    
    # Process regex patterns
    # For backward compatibility, set args.regex based on whether patterns were provided
    args.regex = bool(args.regex_patterns)
    
    # If regex patterns are provided, use them as the keywords
    # Otherwise use the positional arguments
    if args.regex_patterns:
        # Multiple regex patterns will be ANDed together
        args.keywords = args.regex_patterns
        args.multiple_regex_and = True  # Flag to indicate AND logic
    else:
        # Ensure we have keywords from positional args
        if not args.keywords:
            parser.error("Either provide keywords as positional arguments or use -r option(s)")
        args.multiple_regex_and = False
    
    return args, ext_set


"""
Display C++ inheritance hierarchies as tree structures
"""
from typing import List, Dict, Set, Optional, Tuple
from pathlib import Path
import obt.deco as deco
import obt.path
from obt.cpp_database_v2 import CppDatabaseV2
from obt.cpp_entities_v2 import Entity, EntityType
from obt.cpp_search_utils import find_entities_by_name, group_entities_by_namespace

class InheritanceTreeDisplay:
    """Display inheritance hierarchies with tree-like visualization"""
    
    def __init__(self, db: CppDatabaseV2):
        self.db = db
        self.deco = deco.Deco()
        self._cache = {}  # Cache for performance
        
    
    def _build_inheritance_tree_for_entity(self, entity: Entity) -> Dict:
        """Build inheritance tree for a specific entity (prevents namespace conflicts)"""
        tree = {
            'name': entity.short_name,
            'entity': entity,
            'parents': [],
            'children': []
        }
        
        # Get parents (base classes)
        if entity.base_classes:
            for base in entity.base_classes:
                # Clean up base class name (remove 'public', 'private', 'protected')
                base_name = base.replace('public ', '').replace('private ', '').replace('protected ', '').strip()
                # Skip generic base classes that are likely not in our database
                if base_name not in ['Object', 'object']:
                    # Find parent entity and recurse
                    parent_entities = find_entities_by_name(self.db, base_name)
                    if parent_entities:
                        parent_tree = self._build_inheritance_tree_for_entity(parent_entities[0])
                    else:
                        # Parent not found in database - create placeholder
                        parent_tree = {
                            'name': base_name,
                            'entity': None,
                            'parents': [],
                            'children': []
                        }
                    tree['parents'].append(parent_tree)
        
        # Get children (derived classes) - check both short name and canonical name
        derived = self.db.find_derived_classes(entity.canonical_name)
        # Also check short name if different
        if entity.short_name != entity.canonical_name:
            derived.extend(self.db.find_derived_classes(entity.short_name))
            # Remove duplicates
            seen = set()
            unique_derived = []
            for d in derived:
                if d.canonical_name not in seen:
                    seen.add(d.canonical_name)
                    unique_derived.append(d)
            derived = unique_derived
        
        for child_entity in derived:
            child_tree = {
                'name': child_entity.short_name,
                'entity': child_entity,
                'parents': [],  # Don't recurse up from children
                'children': []   # Optionally recurse down
            }
            # Recursively get grandchildren (no namespace filtering)
            child_tree['children'] = self._get_children_trees(
                child_entity, {entity.canonical_name, child_entity.canonical_name})
            tree['children'].append(child_tree)
        
        return tree
    
    def _get_children_trees(self, entity: Entity, visited: Set[str]) -> List[Dict]:
        """Recursively get children, avoiding cycles"""
        if entity.canonical_name in visited:
            return []
        visited.add(entity.canonical_name)
        
        children = []
        # Check both canonical and short names
        derived = self.db.find_derived_classes(entity.canonical_name)
        if entity.short_name != entity.canonical_name:
            derived.extend(self.db.find_derived_classes(entity.short_name))
            # Remove duplicates
            seen = set()
            unique_derived = []
            for d in derived:
                if d.canonical_name not in seen:
                    seen.add(d.canonical_name)
                    unique_derived.append(d)
            derived = unique_derived
            
        for child_entity in derived:
            if child_entity.canonical_name not in visited:
                child_tree = {
                    'name': child_entity.short_name,
                    'entity': child_entity,
                    'parents': [],
                    'children': self._get_children_trees(child_entity, visited.copy())
                }
                children.append(child_tree)
        return children
    
    def _get_children_trees_filtered(self, class_name: str, visited: Set[str], namespace: str) -> List[Dict]:
        """Recursively get children, avoiding cycles and filtering by namespace"""
        if class_name in visited:
            return []
        visited.add(class_name)
        
        children = []
        derived = self.db.find_derived_classes(class_name)
        for child_entity in derived:
            if child_entity.short_name not in visited and child_entity.namespace == namespace:
                child_tree = {
                    'name': child_entity.short_name,
                    'entity': child_entity,
                    'parents': [],
                    'children': self._get_children_trees_filtered(child_entity.short_name, visited.copy(), namespace)
                }
                children.append(child_tree)
        return children
    
    
    def display_tree(self, class_name: str, show_namespaces: bool = True, 
                    show_files: bool = False, root_path: Optional[Path] = None):
        """Display the inheritance tree with formatting"""
        # Find all entities with this name (handles both short and canonical names)
        entities = find_entities_by_name(self.db, class_name)
        
        if not entities:
            print(f"{self.deco.red(f'Class/struct not found: {class_name}')}")
            return
        
        # Group by namespace to separate different base classes
        entities_by_ns = group_entities_by_namespace(entities)
        
        # If only one namespace, use single tree display
        if len(entities_by_ns) == 1:
            # Use the first entity from the single namespace
            entity = list(entities_by_ns.values())[0][0]
            tree = self._build_inheritance_tree_for_entity(entity)
            self._display_single_tree(tree, class_name, show_namespaces, show_files, root_path)
        else:
            # Multiple namespaces - display each separately
            self._display_multiple_trees(entities_by_ns, class_name, show_namespaces, show_files, root_path)
    
    def _display_single_tree(self, tree: Dict, class_name: str, show_namespaces: bool, 
                            show_files: bool, root_path: Optional[Path]):
        """Display a single inheritance tree"""
        if not tree['entity']:
            print(f"{self.deco.red(f'Class/struct not found: {class_name}')}")
            return
        
        # Get the main file path for header
        main_file = None
        if tree['entity'] and tree['entity'].locations:
            main_file = tree['entity'].locations[0].file_path
            # Use obt.path.Path for sanitized path
            main_file = str(obt.path.Path(main_file).sanitized)
        
        # Display header with file path like standard display
        print(f"\n/////////////////////////////////////////////////////////////")
        print(f"// Inheritance Tree for {class_name}")
        if main_file:
            print(f"// {main_file}")
        print(f"/////////\n")
        
        # Display parent hierarchy (ancestors)
        if tree['parents']:
            print(f"\n{self.deco.cyan('BASE CLASSES:')}")
            self._display_parents(tree['parents'], "", True, show_namespaces, show_files, root_path)
        
        # Display the class itself 
        print(f"\n{self.deco.yellow('TARGET:')}")
        ns_str = tree['entity'].namespace if tree['entity'].namespace else "-"
        type_str = "struct" if tree['entity'].entity_type == EntityType.STRUCT else "class"
        name_str = tree['entity'].short_name
        ns_colored = f"\033[48;5;233m\033[38;5;120m{ns_str:^20}\033[m"
        type_colored = f"\033[48;5;236m\033[38;5;51m{type_str:^6}\033[m"
        name_colored = f"\033[48;5;234m\033[38;5;226m{name_str}\033[m"  # Yellow for target
        print(f"{ns_colored}     {type_colored} {name_colored}")
        
        # Display children hierarchy (descendants)
        if tree['children']:
            print(f"\n{self.deco.cyan('DERIVED CLASSES:')}")
            self._display_children(tree['children'], "", True, show_namespaces, show_files, root_path)
        
        # Summary
        parent_count = self._count_ancestors(tree['parents'])
        child_count = self._count_descendants(tree['children'])
        print(f"\n{self.deco.cyan('SUMMARY:')}")
        print(f"  Ancestors: {parent_count}")
        print(f"  Descendants: {child_count}")
    
    def _display_multiple_trees(self, entities_by_ns: Dict, class_name: str, show_namespaces: bool, 
                               show_files: bool, root_path: Optional[Path]):
        """Display multiple inheritance trees grouped by namespace"""
        print(f"\n/////////////////////////////////////////////////////////////")
        print(f"// Multiple {class_name} classes found in different namespaces")
        print(f"/////////\n")
        
        for i, (namespace, entities) in enumerate(sorted(entities_by_ns.items())):
            if i > 0:
                print("\n" + "="*80 + "\n")
            
            # Build tree for the first entity in this namespace
            entity = entities[0]
            qualified_name = f"{namespace}::{class_name}" if namespace != "-" else class_name
            
            print(f"{self.deco.cyan(f'NAMESPACE: {namespace}')}")
            
            # Build inheritance tree for this specific entity
            tree = self._build_inheritance_tree_for_entity(entity)
            
            # Get the main file path for header
            main_file = None
            if entity.locations:
                main_file = entity.locations[0].file_path
                main_file = str(obt.path.Path(main_file).sanitized)
            
            if main_file:
                print(f"// {main_file}")
            
            # Display parent hierarchy (ancestors)
            if tree['parents']:
                print(f"\n{self.deco.cyan('BASE CLASSES:')}")
                self._display_parents(tree['parents'], "", True, show_namespaces, show_files, root_path)
            
            # Display the class itself 
            print(f"\n{self.deco.yellow('TARGET:')}")
            print(f"{self._format_entity(entity, show_namespaces, show_files, root_path, is_target=True)}")
            
            # Display children hierarchy (descendants)
            if tree['children']:
                print(f"\n{self.deco.cyan('DERIVED CLASSES:')}")
                self._display_children(tree['children'], "", True, show_namespaces, show_files, root_path)
            
            # Summary
            parent_count = self._count_ancestors(tree['parents'])
            child_count = self._count_descendants(tree['children'])
            print(f"\n{self.deco.cyan('SUMMARY:')}")
            print(f"  Ancestors: {parent_count}")
            print(f"  Descendants: {child_count}")
    
    def _display_parents(self, parents: List[Dict], prefix: str, is_last: bool,
                        show_namespaces: bool, show_files: bool, root_path: Optional[Path]):
        """Display parent classes with tree structure"""
        for i, parent in enumerate(parents):
            is_last_parent = (i == len(parents) - 1)
            
            # Tree characters
            if prefix == "":
                tree_char = "╰── " if is_last_parent else "├── "
                next_prefix = "    " if is_last_parent else "│   "
            else:
                tree_char = "└── " if is_last_parent else "├── "
                next_prefix = prefix + ("    " if is_last_parent else "│   ")
            
            # Display this parent
            if parent['entity']:
                ns_str = parent['entity'].namespace if parent['entity'].namespace else "-"
                type_str = "struct" if parent['entity'].entity_type == EntityType.STRUCT else "class"
                name_str = parent['entity'].short_name
                ns_colored = f"\033[48;5;233m\033[38;5;120m{ns_str:^20}\033[m"
                type_colored = f"\033[48;5;236m\033[38;5;51m{type_str:^6}\033[m"
                name_colored = f"\033[48;5;234m\033[38;5;231m{name_str}\033[m"
                print(f"{ns_colored} {prefix}{tree_char}{type_colored} {name_colored}")
            else:
                print(f"{'<unknown>':>20} {prefix}{tree_char}??? {parent['name']}")
            
            # Recursively display grandparents
            if parent['parents']:
                self._display_parents(parent['parents'], next_prefix, True, 
                                     show_namespaces, show_files, root_path)
    
    def _display_children(self, children: List[Dict], prefix: str, is_last: bool,
                         show_namespaces: bool, show_files: bool, root_path: Optional[Path]):
        """Display child classes with tree structure"""
        for i, child in enumerate(children):
            is_last_child = (i == len(children) - 1)
            
            # Tree characters
            tree_char = "└── " if is_last_child else "├── "
            next_prefix = prefix + ("    " if is_last_child else "│   ")
            
            # Display this child
            ns_str = child['entity'].namespace if child['entity'].namespace else "-"
            type_str = "struct" if child['entity'].entity_type == EntityType.STRUCT else "class"
            name_str = child['entity'].short_name
            ns_colored = f"\033[48;5;233m\033[38;5;120m{ns_str:^20}\033[m"
            type_colored = f"\033[48;5;236m\033[38;5;51m{type_str:^6}\033[m"
            name_colored = f"\033[48;5;234m\033[38;5;231m{name_str}\033[m"
            print(f"{ns_colored} {prefix}{tree_char}{type_colored} {name_colored}")
            
            # Recursively display grandchildren
            if child['children']:
                self._display_children(child['children'], next_prefix, True,
                                     show_namespaces, show_files, root_path)
    
    def _format_entity(self, entity: Optional[Entity], show_namespaces: bool, 
                      show_files: bool, root_path: Optional[Path], is_target: bool = False) -> str:
        """Format an entity for display matching standard display colors"""
        if not entity:
            return ""
        
        # Build the display string with namespace first
        parts = []
        
        # Namespace column (green on dark bg) 
        ns_str = entity.namespace if entity.namespace else "-"
        # Pad/truncate to fixed width
        ns_display = f"{ns_str:^20}"[:20]
        parts.append(f"\033[48;5;233m\033[38;5;120m{ns_display}\033[m")
        
        # Type column (cyan on dark bg)
        type_str = "struct" if entity.entity_type == EntityType.STRUCT else "class"
        parts.append(f"\033[48;5;236m\033[38;5;51m {type_str:^6} \033[m")
        
        # Name (white on dark bg, or yellow if target)
        name_color = "226" if is_target else "231"
        name_str = entity.short_name
        if entity.is_template and entity.template_params:
            name_str += f"<{entity.template_params}>"
        parts.append(f"\033[48;5;234m\033[38;5;{name_color}m{name_str}\033[m")
        
        return "".join(parts)
    
    def _count_ancestors(self, parents: List[Dict]) -> int:
        """Count total number of ancestor classes"""
        count = len(parents)
        for parent in parents:
            count += self._count_ancestors(parent['parents'])
        return count
    
    def _count_descendants(self, children: List[Dict]) -> int:
        """Count total number of descendant classes"""
        count = len(children)
        for child in children:
            count += self._count_descendants(child['children'])
        return count


#!/usr/bin/env python3

import argparse
from obt import systemd
from obt.deco import Deco
from obt import command

def main():
    parser = argparse.ArgumentParser(description="Restart a systemd service")
    parser.add_argument("service_name", help="Service module name (e.g., ork_logger_http)")

    args = parser.parse_args()
    deco = Deco()

    try:
        # Verify service module exists
        svc = systemd.requires(args.service_name)
        info = svc.info()

        print(deco.inf(f"Restarting service: {args.service_name}"))

        # Use sudo systemctl for system services, systemctl --user for user services
        if info.get('as_system_service', False):
            command.run(["sudo", "systemctl", "restart", f"{args.service_name}.service"])
        else:
            command.run(["systemctl", "--user", "restart", f"{args.service_name}.service"])

    except RuntimeError as e:
        print(deco.err(str(e)))
        exit(1)

if __name__ == "__main__":
    main()




###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from obt import dep, path

###############################################################################

class lapack(dep.StdProvider):
  name = "lapack"
  def __init__(self):
    super().__init__(lapack.name)
    self.declareDep("cmake")
    self._builder = self.createBuilder(dep.CMakeBuilder)

  ########################################################################
  @property
  def _fetcher(self):
    fetcher = dep.GithubFetcher(name=lapack.name,
                                repospec="Reference-LAPACK/lapack",
                                revision="v3.9.0",
                                recursive=False)
    return fetcher
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"README.md").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"liblapack.so").exists()



#!/usr/bin/env python3
"""
Search C++ entity database V2 for Orkid
Fast database-based searches with typedef resolution
"""

import os
import sys
import argparse
import json
import re
from pathlib import Path

# Add obt to path
import obt.path
import obt.deco
from obt.cpp_database_v2 import CppDatabaseV2
from obt.cpp_entities_v2 import EntityType, MemberType, AccessLevel
from obt.cpp_display_v2 import CppEntityDisplayV2
from obt.cpp_search_v2 import search_database, format_json_results
from obt.cpp_argparse_v2 import create_search_parser

deco = obt.deco.Deco()
display = CppEntityDisplayV2()

# Note: search and display helper functions moved to cpp_search_v2.py and cpp_display_v2.py

def add_ork_specific_args(parser):
    """Add Orkid-specific arguments to the parser"""
    # Database selection (optional with default)
    parser.add_argument('--project', '-p', default='orkid',
                       help='Project database to search (default: orkid)')
    
    return parser

def main():
    # Create parser with common arguments and ork-specific arguments
    parser = create_search_parser(
        description='Search C++ entity database V2 for Orkid',
        add_custom_args=add_ork_specific_args
    )
    
    args = parser.parse_args()
    
    # Get database path
    import obt.path as obt_path
    db_path = obt_path.stage() / f"cpp_db_v2_{args.project}.db"
    
    if not db_path.exists():
        print(f"{deco.red(f'Database not found: {db_path}')}")
        print(f"Run ork.cpp.db.build.py to build the database first")
        sys.exit(1)
    
    # Open database
    db = CppDatabaseV2(db_path)
    
    # Output results based on display mode
    if args.display_mode == 'inhtree':
        # Inheritance tree display
        from obt.cpp_inheritance_tree import InheritanceTreeDisplay
        tree_display = InheritanceTreeDisplay(db)
        
        # For inheritance tree, we need a specific class name
        if args.pattern:
            tree_display.display_tree(args.pattern, show_namespaces=True, show_files=True)
        else:
            print(f"{deco.red('Please specify a class/struct name for inheritance tree display')}")
            sys.exit(1)
    elif args.display_mode == 'details':
        # Class details display
        from obt.cpp_class_details import ClassDetailsDisplay
        details_display = ClassDetailsDisplay(db)
        
        # For class details, we need a specific class name
        if args.pattern:
            details_display.display_details(args.pattern, show_files=True)
        else:
            print(f"{deco.red('Please specify a class/struct name for details display')}")
            sys.exit(1)
    else:
        # Standard mode - perform search first
        results = search_database(db, args)
        
        if not results:
            print(f"{deco.yellow('No results found')}")
            sys.exit(0)
        
        # Check if we're in files mode (results are dicts not Entity objects)
        if results and isinstance(results[0], dict):
            # File listing mode
            print(f"Found {len(results)} files:")
            print("=" * 80)
            for file_info in results:
                size_kb = file_info['file_size'] / 1024 if file_info.get('file_size') else 0
                relative_path = file_info.get('relative_path', file_info['file_path'])
                print(f"{relative_path:<60} {size_kb:>8.1f} KB")
        elif args.json:
            # JSON output for tool integration
            json_results = format_json_results(results)
            print(json.dumps(json_results, indent=2))
        else:
            # Standard file-based display
            # Pass the base class if we're showing derived classes
            show_base = args.derived_from if hasattr(args, 'derived_from') and args.derived_from else None
            display.display_entities(results, root_path=None, sepfiles=args.sepfiles, show_base_class=show_base)
            
            if args.limit > 0 and len(results) == args.limit:
                print(f"\n{deco.yellow(f'Results limited to {args.limit}. Use --limit to see more.')}")

if __name__ == '__main__':
    main()


#!/usr/bin/env python3
"""
C++ Access Analyzer - Second-pass analysis for tracking entity accesses
Analyzes C++ source code to track reads, writes, and calls to entities
"""

from dataclasses import dataclass
from typing import Optional, List, Dict, Any
from pathlib import Path
import tree_sitter_cpp as ts_cpp
from tree_sitter import Language, Parser, Node

@dataclass
class AccessRecord:
    """Record of an entity access"""
    accessed_entity: str  # Canonical name or partial name
    access_type: str  # 'read', 'write', 'call'  
    file_path: str
    line_number: int
    column_number: int
    accessing_function: Optional[str] = None  # Function performing the access
    context_snippet: Optional[str] = None  # Code snippet for context
    
class CppAccessAnalyzer:
    """
    Second-pass analyzer for tracking entity accesses in C++ code
    Runs after initial parsing to track reads, writes, and calls
    """
    
    def __init__(self, db=None, verbose: bool = False, debug: bool = False):
        """
        Initialize the access analyzer
        
        Args:
            db: CppDatabaseV2 instance with populated entities
            verbose: Enable verbose output
            debug: Enable debug output
        """
        self.db = db
        self.verbose = verbose
        self.debug = debug
        self.parser = Parser()
        self.parser.set_language(Language(ts_cpp.language(), "cpp"))
        
        # Access tracking state
        self.accesses = []
        self.current_file = None
        self.current_function_context = None
        self.current_class_context = None  # Track current class for member resolution
        self.namespace_stack = []
        self.class_stack = []  # Stack for nested classes
        
    def analyze_file(self, filepath: Path, source_code: Optional[str] = None) -> List[AccessRecord]:
        """
        Analyze a file for entity accesses
        
        Args:
            filepath: Path to the C++ file
            source_code: Optional source code (if not provided, will read from file)
            
        Returns:
            List of AccessRecord objects
        """
        self.current_file = filepath
        self.accesses = []
        self.namespace_stack = []
        self.current_function_context = None
        
        # Read source if not provided
        if source_code is None:
            source_code = filepath.read_text(encoding='utf-8', errors='ignore')
        
        # Parse with tree-sitter
        tree = self.parser.parse(source_code.encode('utf-8'))
        source_bytes = source_code.encode('utf-8')
        
        # Analyze the AST
        self._analyze_node(tree.root_node, source_bytes)
        
        return self.accesses
    
    def _analyze_node(self, node: Node, source: bytes):
        """Recursively analyze AST nodes"""
        
        if self.debug and node.type in ['function_definition', 'call_expression', 'field_expression']:
            print(f"DEBUG: Found {node.type} at line {source[:node.start_byte].count(b'\\n') + 1}")
        
        # Track namespace context
        if node.type == 'namespace_definition':
            self._enter_namespace(node, source)
            for child in node.children:
                self._analyze_node(child, source)
            self._exit_namespace()
            return
        
        # Track class/struct context
        elif node.type in ['class_specifier', 'struct_specifier']:
            self._analyze_class_definition(node, source)
            return
        
        # Track function context
        elif node.type == 'function_definition':
            self._analyze_function_definition(node, source)
            return
        
        # Look for expressions that access entities
        elif node.type == 'expression_statement':
            self._analyze_expression_statement(node, source)
        
        elif node.type == 'return_statement':
            self._analyze_return_statement(node, source)
        
        elif node.type == 'if_statement':
            self._analyze_if_statement(node, source)
        
        elif node.type == 'for_statement' or node.type == 'while_statement':
            self._analyze_loop_statement(node, source)
        
        elif node.type == 'declaration':
            self._analyze_declaration(node, source)
        
        # Recurse for other node types
        else:
            for child in node.children:
                self._analyze_node(child, source)
    
    def _enter_namespace(self, node: Node, source: bytes):
        """Enter a namespace scope"""
        for child in node.children:
            if child.type == 'identifier' or child.type == 'namespace_identifier':
                namespace_name = self._extract_text(child, source)
                self.namespace_stack.append(namespace_name)
                break
    
    def _exit_namespace(self):
        """Exit a namespace scope"""
        if self.namespace_stack:
            self.namespace_stack.pop()
    
    def _analyze_class_definition(self, node: Node, source: bytes):
        """Analyze a class/struct definition"""
        class_name = None
        
        # Extract class name
        for child in node.children:
            if child.type == 'type_identifier':
                class_name = self._extract_text(child, source)
                break
        
        if class_name:
            # Build full qualified name
            full_name = '::'.join(self.namespace_stack + [class_name]) if self.namespace_stack else class_name
            
            # Enter class context
            self.class_stack.append(full_name)
            old_class = self.current_class_context
            self.current_class_context = full_name
            
            # Analyze class body
            for child in node.children:
                if child.type == 'field_declaration_list':
                    # This contains the class members
                    for member_child in child.children:
                        self._analyze_node(member_child, source)
            
            # Exit class context
            self.class_stack.pop()
            self.current_class_context = old_class
    
    def _analyze_function_definition(self, node: Node, source: bytes):
        """Analyze a function definition and its body"""
        print(f"ENTERING FUNCTION at line {source[:node.start_byte].count(b'\n') + 1}")
        
        # Extract function name for context
        function_name = None
        compound_statement = None
        
        for child in node.children:
            if child.type == 'function_declarator':
                function_name = self._extract_function_name(child, source)
                print(f"  Function name: {function_name}")
            elif child.type == 'compound_statement':
                compound_statement = child
                print(f"  Found compound_statement")
        
        if function_name:
            # Build full qualified name with namespace
            if self.namespace_stack:
                full_name = '::'.join(self.namespace_stack) + '::' + function_name
            else:
                full_name = function_name
            
            print(f"  Full function name: {full_name}")
            
            # Analyze function body with this context
            if compound_statement:
                print(f"  ANALYZING BODY with class context: {self.current_class_context}")
                old_func_context = self.current_function_context
                old_class_context = self.current_class_context
                self.current_function_context = full_name
                # Note: current_class_context may have been set by qualified_identifier extraction
                self._analyze_compound_statement(compound_statement, source)
                self.current_function_context = old_func_context
                self.current_class_context = old_class_context
                print(f"  DONE WITH BODY")
    
    def _extract_function_name(self, declarator_node: Node, source: bytes) -> Optional[str]:
        """Extract function name from function_declarator node"""
        for child in declarator_node.children:
            if child.type == 'qualified_identifier':
                # This is like ClassName::methodName
                text = self._extract_text(child, source)
                # Also extract the class name for context
                parts = text.split('::')
                if len(parts) >= 2:
                    # Set class context from the qualified name
                    self.current_class_context = '::'.join(parts[:-1])
                return text
            elif child.type in ['identifier', 'field_identifier', 'destructor_name']:
                return self._extract_text(child, source)
            elif child.type == 'operator_name':
                return self._extract_operator_name(child, source)
        return None
    
    def _extract_operator_name(self, node: Node, source: bytes) -> str:
        """Extract operator name like operator+"""
        text = self._extract_text(node, source)
        # Normalize operator names
        return text.replace(' ', '')
    
    def _analyze_compound_statement(self, node: Node, source: bytes):
        """Analyze a compound statement (block)"""
        for child in node.children:
            if child.type not in ['{', '}']:
                self._analyze_node(child, source)
    
    def _analyze_expression_statement(self, node: Node, source: bytes):
        """Analyze an expression statement"""
        for child in node.children:
            if child.type != ';':
                self._analyze_expression(child, source)
    
    def _analyze_return_statement(self, node: Node, source: bytes):
        """Analyze a return statement"""
        for child in node.children:
            if child.type not in ['return', ';']:
                self._analyze_expression(child, source)
    
    def _analyze_if_statement(self, node: Node, source: bytes):
        """Analyze an if statement"""
        for child in node.children:
            if child.type == 'parenthesized_expression':
                self._analyze_expression(child, source)
            elif child.type == 'compound_statement':
                self._analyze_compound_statement(child, source)
            elif child.type not in ['if', 'else']:
                self._analyze_node(child, source)
    
    def _analyze_loop_statement(self, node: Node, source: bytes):
        """Analyze a for or while loop"""
        for child in node.children:
            if child.type == 'parenthesized_expression':
                self._analyze_expression(child, source)
            elif child.type == 'compound_statement':
                self._analyze_compound_statement(child, source)
            elif child.type not in ['for', 'while', 'do']:
                self._analyze_node(child, source)
    
    def _analyze_declaration(self, node: Node, source: bytes):
        """Analyze a declaration which may have initializers"""
        for child in node.children:
            if child.type == 'init_declarator':
                self._analyze_init_declarator(child, source)
    
    def _analyze_init_declarator(self, node: Node, source: bytes):
        """Analyze variable initialization"""
        for child in node.children:
            if child.type not in ['identifier', '=', 'pointer_declarator']:
                self._analyze_expression(child, source)
    
    def _analyze_expression(self, node: Node, source: bytes, is_write: bool = False):
        """
        Analyze an expression for entity accesses
        
        Args:
            node: The expression node
            source: Source code bytes
            is_write: Whether this expression is being written to
        """
        if node.type == 'field_expression':
            self._analyze_field_expression(node, source, is_write)
        
        elif node.type == 'call_expression':
            self._analyze_call_expression(node, source)
        
        elif node.type == 'assignment_expression':
            self._analyze_assignment_expression(node, source)
        
        elif node.type == 'parenthesized_expression':
            # Unwrap parentheses
            for child in node.children:
                if child.type not in ['(', ')']:
                    self._analyze_expression(child, source, is_write)
        
        elif node.type == 'binary_expression':
            # Analyze both sides of binary expression
            for child in node.children:
                if child.type not in ['+', '-', '*', '/', '%', '==', '!=', '<', '>', '<=', '>=', '&&', '||', '&', '|', '^', '<<', '>>']:
                    self._analyze_expression(child, source)
        
        elif node.type == 'unary_expression':
            # Analyze the operand
            for child in node.children:
                if child.type not in ['!', '~', '-', '+', '++', '--', '*', '&']:
                    # Dereference (*) is a read, address-of (&) doesn't access the value
                    if node.children[0].type == '*':
                        self._analyze_expression(child, source)
                    elif node.children[0].type != '&':
                        self._analyze_expression(child, source, is_write)
        
        elif node.type == 'update_expression':
            # ++ or -- operators
            for child in node.children:
                if child.type not in ['++', '--']:
                    self._analyze_expression(child, source, is_write=True)
        
        elif node.type == 'subscript_expression':
            # Array access like arr[i]
            for child in node.children:
                if child.type not in ['[', ']']:
                    self._analyze_expression(child, source, is_write)
        
        elif node.type == 'conditional_expression':
            # Ternary operator
            for child in node.children:
                if child.type not in ['?', ':']:
                    self._analyze_expression(child, source)
        
        elif node.type == 'cast_expression':
            # Type cast
            for child in node.children:
                if child.type not in ['(', ')'] and not child.type.endswith('_type'):
                    self._analyze_expression(child, source, is_write)
        
        elif node.type == 'sizeof_expression':
            # sizeof doesn't actually access the value
            pass
        
        elif node.type == 'identifier':
            # Bare identifier - could be member, local, or global
            var_name = self._extract_text(node, source)
            
            # If we're in a class context, this might be a member variable
            # Record it with the class context for later resolution
            if self.current_class_context:
                # Store as "ClassName::memberName" for resolution
                entity_name = f"{self.current_class_context}::{var_name}"
            else:
                entity_name = var_name
            
            access_type = 'write' if is_write else 'read'
            self._record_access(entity_name, access_type, node, source)
        
        elif node.type == 'qualified_identifier':
            # Fully qualified name like Class::member or std::cout
            qual_name = self._extract_text(node, source)
            access_type = 'write' if is_write else 'read'
            self._record_access(qual_name, access_type, node, source)
        
        else:
            # Recurse for other expression types
            for child in node.children:
                if child.type not in ['(', ')', '{', '}', ';', ',']:
                    self._analyze_expression(child, source, is_write)
    
    def _analyze_field_expression(self, node: Node, source: bytes, is_write: bool = False):
        """Analyze field access like obj.member or obj->member"""
        field_name = None
        object_expr = None
        
        for child in node.children:
            if child.type == 'field_identifier':
                field_name = self._extract_text(child, source)
            elif child.type not in ['.', '->']:
                object_expr = child
        
        if field_name:
            # Record the field access
            access_type = 'write' if is_write else 'read'
            self._record_access(field_name, access_type, node, source)
        
        # Analyze the object expression (it's being read to access the field)
        if object_expr:
            self._analyze_expression(object_expr, source, is_write=False)
    
    def _analyze_call_expression(self, node: Node, source: bytes):
        """Analyze function/method calls"""
        function_name = None
        argument_list = None
        
        for child in node.children:
            if child.type == 'identifier':
                function_name = self._extract_text(child, source)
            elif child.type == 'qualified_identifier':
                function_name = self._extract_text(child, source)
            elif child.type == 'field_expression':
                # Method call like obj.method()
                self._analyze_field_expression_for_call(child, source)
            elif child.type == 'argument_list':
                argument_list = child
        
        if function_name:
            self._record_access(function_name, 'call', node, source)
        
        # Analyze arguments
        if argument_list:
            for child in argument_list.children:
                if child.type not in ['(', ')', ',']:
                    self._analyze_expression(child, source)
    
    def _analyze_field_expression_for_call(self, node: Node, source: bytes):
        """Analyze field expression that's being called as a method"""
        method_name = None
        object_expr = None
        
        for child in node.children:
            if child.type == 'field_identifier':
                method_name = self._extract_text(child, source)
            elif child.type not in ['.', '->']:
                object_expr = child
        
        if method_name:
            self._record_access(method_name, 'call', node, source)
        
        # The object is being read to call the method
        # This includes nested field expressions like rval._frustum.set()
        # where we need to record both the access to _frustum and the call to set
        if object_expr:
            self._analyze_expression(object_expr, source, is_write=False)
    
    def _analyze_assignment_expression(self, node: Node, source: bytes):
        """Analyze assignment to properly classify reads and writes"""
        left_side = True
        
        for child in node.children:
            if child.type in ['=', '+=', '-=', '*=', '/=', '%=', '&=', '|=', '^=', '<<=', '>>=']:
                left_side = False
            else:
                # Left side is written to, right side is read from
                self._analyze_expression(child, source, is_write=left_side)
    
    def _record_access(self, entity_name: str, access_type: str, node: Node, source: bytes):
        """Record an entity access"""
        line_num = source[:node.start_byte].count(b'\n') + 1
        col_num = node.start_point[1] + 1
        
        # Extract a small context snippet
        start = max(0, node.start_byte - 20)
        end = min(len(source), node.end_byte + 20)
        context = source[start:end].decode('utf-8', errors='ignore').strip()
        
        access = AccessRecord(
            accessed_entity=entity_name,
            access_type=access_type,
            file_path=str(self.current_file),
            line_number=line_num,
            column_number=col_num,
            accessing_function=self.current_function_context,
            context_snippet=context
        )
        
        self.accesses.append(access)
        
        # ALWAYS LOG DETECTIONS
        print(f"DETECTED: {access_type:5} '{entity_name}' at {self.current_file}:{line_num}:{col_num}")
        
        if self.verbose:
            print(f"  Context: {context[:50]}")
    
    def _extract_text(self, node: Node, source: bytes) -> str:
        """Extract text from a node"""
        return source[node.start_byte:node.end_byte].decode('utf-8', errors='ignore')
    
    def store_accesses_to_db(self, accesses: List[AccessRecord]):
        """
        Store access records to the database
        
        Args:
            accesses: List of AccessRecord objects to store
        """
        if not self.db:
            raise ValueError("No database connection available")
        
        for access in accesses:
            # Try to resolve the entity in the database
            # This is where we'd look up the canonical name from partial names
            # For now, we'll store with the name we found
            
            # TODO: Implement entity resolution logic
            # - Look up entity by name
            # - Handle qualified vs unqualified names
            # - Resolve member access to full canonical path
            
            # For now, just store the raw access
            # The database will need methods to resolve these later
            pass


#!/usr/bin/env python3

import argparse
import os
from pathlib import Path
from obt import systemd
from obt.systemd_helpers import create_systemd_service
from obt.deco import Deco

def main():
    parser = argparse.ArgumentParser(description="Create systemd user service for OBT/Uni projects")
    parser.add_argument("service_name", help="Service module name (e.g., ork_logger_http)")
    parser.add_argument("--dest", help="Destination folder for service files (default: ${OBT_STAGE}/systemd/<service_name>)")
    parser.add_argument("--command", help="Override command from service module")
    parser.add_argument("--deploy", action="store_true", help="Deploy to ~/.config/systemd/user/")
    parser.add_argument("--start", action="store_true", help="Restart the service (requires --deploy)")
    parser.add_argument("--enable", action="store_true", help="Enable the service to start on boot (requires --deploy)")

    args = parser.parse_args()
    deco = Deco()

    try:
        # Load service module
        svc = systemd.requires(args.service_name)
        info = svc.info()

        # Use command from module unless overridden
        command_str = args.command if args.command else info['command']

        # Default dest to ${OBT_STAGE}/systemd/<service_name>
        if args.dest:
            dest_folder = args.dest
        else:
            obt_stage = os.environ.get("OBT_STAGE")
            if not obt_stage:
                print(deco.err("OBT_STAGE environment variable not set"))
                exit(1)
            dest_folder = str(Path(obt_stage) / "systemd" / args.service_name)

        # Create the service
        create_systemd_service(
            args.service_name,
            dest_folder,
            command_str,
            deploy=args.deploy,
            start=args.start,
            enable=args.enable,
            requires=info.get('requires', []),
            prefers=info.get('prefers', []),
            after=info.get('after', []),
            before=info.get('before', []),
            requires_deps=info.get('requires_deps', []),
            requires_dockers=info.get('requires_dockers', []),
            requires_pips=info.get('requires_pips', []),
            requires_tty=info.get('requires_tty', None),
            as_system_service=info.get('as_system_service', False)
        )

    except RuntimeError as e:
        print(deco.err(str(e)))
        exit(1)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
Build C++ entity database V2 for any C++ codebase
General-purpose tool with directory argument
Uses normalized database design with tree-sitter parsing
"""

import os
import sys
import argparse
from pathlib import Path
import time

# Add obt to path
import obt.path
import obt.deco
from obt.cpp_parser_descent import RecursiveDescentCppParser
from obt.cpp_database_v2 import CppDatabaseV2
from obt.cpp_entities_v2 import EntityType

deco = obt.deco.Deco()

def find_source_files(paths, extensions=None):
    """Find all C++ source files in given paths"""
    if extensions is None:
        # C++ extensions
        extensions = {'.h', '.hpp', '.hxx', '.H', '.hh', 
                     '.c', '.cpp', '.cxx', '.cc', '.C',
                     '.inl', '.inc'}
    
    files = []
    for path in paths:
        if path.is_file():
            if path.suffix in extensions:
                files.append(path)
        elif path.is_dir():
            for ext in extensions:
                files.extend(path.rglob(f'*{ext}'))
    
    return sorted(set(files))

def build_database(db_path, source_paths, verbose=False, show_progress=True):
    """Build the C++ entity database"""
    
    # Create database
    db = CppDatabaseV2(db_path)
    
    # Clear existing data
    if verbose:
        print(f"{deco.yellow('Clearing existing database...')}")
    db.clear_database()
    
    # Create parser
    parser = RecursiveDescentCppParser()
    
    # Find all source files
    if verbose:
        print(f"{deco.yellow('Finding source files...')}")
    
    files = find_source_files(source_paths)
    total_files = len(files)
    
    if verbose:
        print(f"{deco.green(f'Found {total_files} source files')}")
    
    # Parse files and add to database
    start_time = time.time()
    parsed_count = 0
    entity_count = 0
    error_count = 0
    
    for i, file_path in enumerate(files):
        if show_progress:
            progress = (i + 1) / total_files * 100
            print(f"\r{deco.cyan(f'Progress: {progress:.1f}%')} ({i+1}/{total_files})", end='', flush=True)
        
        try:
            # Parse file
            entities = parser.parse_file(file_path)
            
            # Add entities to database
            for entity in entities:
                db.add_entity(entity)
                entity_count += 1
            
            # Track parsed file
            db.add_file(file_path)
            parsed_count += 1
            
        except Exception as e:
            error_count += 1
            if verbose:
                print(f"\n{deco.red(f'Error parsing {file_path}: {e}')}")
    
    if show_progress:
        print()  # New line after progress
    
    # Get statistics
    elapsed = time.time() - start_time
    stats = db.get_statistics()
    
    # Print summary
    print(f"\n{deco.green('=== Build Complete ===')}")
    print(f"Database: {db_path}")
    print(f"Time: {elapsed:.2f} seconds")
    print(f"Files parsed: {parsed_count}/{total_files}")
    if error_count > 0:
        print(f"Errors: {deco.red(str(error_count))}")
    print(f"\n{deco.cyan('Entity counts:')}")
    print(f"  Total entities: {stats.get('total_entities', 0)}")
    print(f"  Classes: {stats.get('entities_class', 0)}")
    print(f"  Structs: {stats.get('entities_struct', 0)}")
    print(f"  Functions: {stats.get('entities_function', 0)}")
    print(f"  Enums: {stats.get('entities_enum', 0)}")
    print(f"  Typedefs: {stats.get('entities_typedef', 0)}")
    print(f"  Templates: {stats.get('template_entities', 0)}")
    
    return db

def main():
    parser = argparse.ArgumentParser(
        description='Build C++ entity database V2 for any C++ codebase'
    )
    
    parser.add_argument('directory', nargs='*',
                       help='Directory or directories to scan for C++ files')
    
    parser.add_argument('--project', '-p',
                       required=True,
                       help='Project name for database')
    
    parser.add_argument('-v', '--verbose', action='store_true',
                       help='Verbose output')
    
    parser.add_argument('--no-progress', action='store_true',
                       help='Disable progress indicator')
    
    parser.add_argument('--stats-only', action='store_true',
                       help='Show database statistics without rebuilding')
    
    args = parser.parse_args()
    
    # Get database path from project name
    import obt.path as obt_path
    db_path = obt_path.stage() / f"cpp_db_v2_{args.project}.db"
    
    # If stats only, show stats and exit
    if args.stats_only:
        if not db_path.exists():
            print(f"{deco.red(f'Database not found: {db_path}')}")
            print(f"Project: {args.project}")
            sys.exit(1)
        
        db = CppDatabaseV2(db_path)
        stats = db.get_statistics()
        
        print(f"{deco.green('=== Database Statistics ===')}")
        print(f"Database: {db_path}")
        print(f"\n{deco.cyan('Entity counts:')}")
        print(f"  Total entities: {stats.get('total_entities', 0)}")
        print(f"  Classes: {stats.get('entities_class', 0)}")
        print(f"  Structs: {stats.get('entities_struct', 0)}")
        print(f"  Functions: {stats.get('entities_function', 0)}")
        print(f"  Enums: {stats.get('entities_enum', 0)}")
        print(f"  Typedefs: {stats.get('entities_typedef', 0)}")
        print(f"  Templates: {stats.get('template_entities', 0)}")
        print(f"  Files: {stats.get('total_files', 0)}")
        
        # Show file breakdown by extension
        file_extensions = ['.h', '.hpp', '.hh', '.hxx', '.c', '.cpp', '.cc', '.cxx', '.inl', '.inc']
        file_breakdown = []
        for ext in file_extensions:
            ext_key = ext.lstrip('.')
            count = stats.get(f'files_{ext_key}', 0)
            if count > 0:
                size_mb = stats.get(f'files_{ext_key}_size_mb', 0)
                file_breakdown.append((ext, count, size_mb))
        
        if file_breakdown:
            print(f"    Breakdown by type:")
            for ext, count, size_mb in file_breakdown:
                if size_mb > 0:
                    print(f"      {ext:<5} {count:>4} files ({size_mb:.1f} MB)")
                else:
                    print(f"      {ext:<5} {count:>4} files")
        
        print(f"\n{deco.cyan('Other stats:')}")
        print(f"  Total locations: {stats.get('total_locations', 0)}")
        print(f"  Total members: {stats.get('total_members', 0)}")
        if stats.get('files_with_preprocessed', 0) > 0:
            print(f"  Files with preprocessed: {stats.get('files_with_preprocessed', 0)}")
        if stats.get('total_source_size_mb', 0) > 0:
            print(f"  Total source size: {stats.get('total_source_size_mb', 0)} MB")
        sys.exit(0)
    
    # Get source paths from command line
    if not args.directory:
        print(f"{deco.red('Error: No directories specified')}")
        sys.exit(1)
    
    source_paths = []
    for dir_arg in args.directory:
        path = Path(dir_arg).resolve()
        if path.exists():
            source_paths.append(path)
        else:
            print(f"{deco.red(f'Error: Path not found: {dir_arg}')}")
            sys.exit(1)
    
    if args.verbose:
        print(f"{deco.yellow('Source paths:')}")
        for p in source_paths:
            print(f"  {p}")
    
    # Build database
    build_database(
        db_path,
        source_paths,
        verbose=args.verbose,
        show_progress=not args.no_progress
    )

if __name__ == '__main__':
    main()


###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from obt import dep, path
###############################################################################
class blosc(dep.StdProvider):
  name = "blosc"
  def __init__(self):
    super().__init__(blosc.name)
    self.declareDep("cmake")
    self._builder = self.createBuilder(dep.CMakeBuilder)
    self._builder._cmakeenv = {
      "BUILD_SHARED_LIBS": "ON"
    }

  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=blosc.name,
                             repospec="tweakoz/c-blosc",
                             revision="v1.21.6",
                             recursive=False)
  ########################################################################

  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()

  def areRequiredBinaryFilesPresent(self):
    return path.decorate_obt_lib("blosc").exists()



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os

import obt.deco
from obt import log

deco = obt.deco.Deco()

###########################################

def set(key,val):

  log.output(deco.orange("set")+" var<" + deco.key(str(key))+"> to <" + deco.path(val) + ">")
  os.environ[str(key)] = str(val)

###########################################

def prepend(key,val,dedupe=True):
  val = os.path.normpath(str(val))
  if False==(str(key) in os.environ):
    set(key,str(val))
  else:
    if dedupe:
      prev = os.environ[key]
      if str(val) in prev:
        return
    prev = os.environ[key]
    newv = str(val) + ":" + prev
    os.environ[str(key)] = newv
    log_str = deco.magenta("prepend")+" var<" + deco.key(key) 
    log_str += "> -> <" + deco.val(str(val)) + ":" + deco.path(prev) + ">"
    log.output(log_str)

###########################################

def append(key,val,dedupe=True):
  val = os.path.normpath(str(val))
  if(str(val)==""):
    return
  prev_val_has_length = True
  key_present = str(key) in os.environ
  if key_present:
    prev = os.environ[str(key)]
    prev_val_has_length = len(str(prev))>0
  if (not key_present) or (not prev_val_has_length):
    set(key,val)
    return
  else:
    if dedupe:
      prev = os.environ[key]
      if str(val) in prev:
        return
    #CHK = (str(key) in os.environ.keys())
    prev = os.environ[str(key)]
    newv = prev + ":" + str(val)
    os.environ[str(key)] = newv
    #print("prev<%s:%d:%s>"%(CHK,len(prev),prev))
    #print(os.environ)
    log_str = deco.cyan("append")+" var<" + deco.key(key) 
    log_str += "> -> <" + deco.path(prev) + ":" + deco.val(str(val)) + ">"
    log.output(log_str)

###########################################

def is_set(key):
  return str(key) in os.environ



from obt import dep, path, command, docker, wget, pathtools, host
from obt.deco import Deco
import obt.module
import time, re, socket, os, sys
from pathlib import Path
deco = obt.deco.Deco()

this_path = os.path.realpath(__file__)
this_dir = Path(os.path.dirname(this_path))

###############################################################################

class dockerinfo:
  ###############################################
  def __init__(self):
    super().__init__()
    self.type = docker.Type.COMPOSITE # use docker-compose
    self._name = "ssltest"
    self._manifest_path = path.manifests_root()/self._name
  ###############################################
  # build the docker images
  ###############################################
  def build(self, build_args):
    os.system("rm -f %s" % str(self._manifest_path))
    assert(build_args != None)
    os.chdir(str(this_dir))
    chain = command.chain()
    chain.run(["pbin/build.py"] + build_args)
    OK = chain.ok()
    if OK:
      os.system("touch %s" % str(self._manifest_path))
    return OK
  ###############################################
  # kill active docker containers
  ###############################################
  def kill(self):
    os.chdir(str(this_dir))
    command.run(["docker", "compose", "down"])
  ###############################################
  # launch docker containers
  ###############################################
  def launch(self, launch_args, environment=None, mounts=None):
    os.chdir(this_dir)
    command.run(["docker", "compose", "up"])
  ###############################################
  # show logs
  ###############################################
  def logs(self, service=None):
    os.chdir(this_dir)
    if service:
      command.run(["docker", "compose", "logs", "-f", service])
    else:
      command.run(["docker", "compose", "logs", "-f"])
  ###############################################
  # information dictionary
  ###############################################
  def info(self):
    return {
      "name": "ssltest",
      "description": "Example of SSL-wrapping an unprotected ZMQ server",
      "manifest": str(self._manifest_path),
    }


"""
Stack-based C++ Access Analyzer
Implements context-aware AST traversal to track entity accesses

TODO: Type-Aware Method Call Tracking
--------------------------------------
Current limitation: When tracking method calls like `ctx->SetDisplayMode(mode)`, 
we record the access to the method name but cannot determine:
1. The actual type of `ctx` (could be Context* or ContextGL*)
2. Which overload of SetDisplayMode is being called (depends on type of `mode`)
3. The correct method entity in the inheritance hierarchy

Future enhancement: Integrate libclang for type-aware access tracking
- Use libclang to get exact types of variables and expressions
- Resolve method calls to specific overloads based on argument types  
- Track both the object being called AND the specific method entity
- Handle virtual dispatch correctly

Implementation approach:
1. Keep tree-sitter for fast structural parsing
2. Add optional libclang pass for type resolution when compile_commands.json available
3. For each method call, record TWO accesses:
   - READ access to the object variable (if it's a member)
   - CALL access to the specific method entity (with correct overload)

See test_class_parsing_validation_clang.py for existing libclang integration example.
"""

from dataclasses import dataclass
from enum import Enum
from typing import List, Optional, Dict, Any, Set, Tuple
from pathlib import Path
import json
import tree_sitter as ts
import tree_sitter_cpp as ts_cpp
from tree_sitter import Language, Parser, Node
from obt.cpp_database_v2 import CppDatabaseV2

class StackTreeNode:
    """Wrapper for tree-sitter nodes that tracks parent relationships"""
    def __init__(self, ts_node: Node, parent: Optional['StackTreeNode'] = None):
        self.node = ts_node  # The actual tree-sitter node
        self.parent = parent  # Our parent StackTreeNode
        self._children = None  # Cache children
    
    @property
    def type(self):
        return self.node.type
    
    @property
    def text(self):
        return self.node.text
    
    @property
    def start_point(self):
        return self.node.start_point
    
    @property
    def end_point(self):
        return self.node.end_point
    
    @property
    def children(self):
        """Wrap children on demand with self as parent"""
        if self._children is None:
            self._children = [StackTreeNode(child, parent=self) for child in self.node.children]
        return self._children
    
    def get_ancestor(self, levels_up: int = 1) -> Optional['StackTreeNode']:
        """Walk up the tree by specified levels"""
        current = self
        for _ in range(levels_up):
            if current.parent:
                current = current.parent
            else:
                return None
        return current

class AccessType(Enum):
    """Types of entity access"""
    READ = "read"
    WRITE = "write"
    CALL = "call"
    ADDRESS_OF = "address_of"
    READ_WRITE = "read_write"  # For ++ and +=
    DEF = "def"  # Definition/declaration
    IMPL = "impl"  # Implementation/instantiation (static members, templates, etc.)


@dataclass
class Access:
    """Represents a detected access"""
    raw_identifier: str
    access_type: AccessType
    file_path: Path
    trimmed_line: int
    original_line: int
    column: int
    context_function: Optional[str] = None
    context_class: Optional[str] = None
    context_namespace: Optional[str] = None
    context_snippet: Optional[str] = None
    entity_id: Optional[int] = None
    member_id: Optional[int] = None

class StackBasedAccessAnalyzer:
    """Stack-based analyzer for C++ entity accesses"""
    
    def __init__(self, db_path: str, track_operators: bool = False):
        self.db = CppDatabaseV2(db_path)
        self.track_operators = track_operators
        self.accesses: List[Access] = []
        self.current_file: Optional[Path] = None
        self.line_mapping: Optional[Dict[int, int]] = None
        self.source_bytes: Optional[bytes] = None
        
        # Context tracking
        self.current_function: Optional[str] = None
        self.current_class: Optional[str] = None
        self.current_namespace: Optional[str] = None
        
        # Initialize parser
        self.parser = Parser()
        self.parser.set_language(Language(ts_cpp.language(), "cpp"))
        
    def analyze_file(self, file_path: Path, trimmed_source: str, 
                     line_mapping: Dict[int, int]) -> List[Access]:
        """
        Main entry point for analyzing a file
        
        Args:
            file_path: Path to the source file
            trimmed_source: The trimmed source code
            line_mapping: Maps trimmed line numbers to original line numbers
            
        Returns:
            List of detected accesses
        """
        self.current_file = file_path
        self.line_mapping = line_mapping
        self.accesses = []
        self.source_bytes = trimmed_source.encode()
        
        # Parse the source
        tree = self.parser.parse(self.source_bytes)
        
        # Wrap root node and analyze
        wrapped_root = StackTreeNode(tree.root_node, parent=None)
        self._analyze_node(wrapped_root)
        
        return self.accesses
    
    def _analyze_node(self, node: StackTreeNode):
        """
        Single traversal - node already has parent tracking
        """
        # Update namespace/class/function context
        self._update_context(node)
            
        
        # Process terminal nodes (actual accesses and declarations)
        if node.type in ['identifier', 'field_identifier']:
            # Check if this is being declared
            if self._is_being_declared(node):
                # Track as definition
                self._record_access(node, AccessType.DEF)
            else:
                access_type = self._determine_access_type(node)
                if access_type:  # None means don't track (just navigation)
                    self._record_access(node, access_type)
        # Note: qualified_identifier is NOT a terminal - its children will be processed by recursion
        
        # ALWAYS recurse to children - this is orthogonal to pattern matching
        for child in node.children:
            self._analyze_node(child)
    
    def _determine_access_type(self, node: StackTreeNode) -> Optional[AccessType]:
        """
        Determine if and how to track this access.
        Returns None if this is just navigation (not an endpoint).
        """
        # Check if we're an endpoint or just part of the path
        if not self._is_endpoint(node):
            return None
        
        parent = node.parent
        if not parent:
            return AccessType.READ  # Default for top-level identifiers
        
        grandparent = node.get_ancestor(2)
        
        # Check immediate parent type
        if parent.type == 'pointer_expression':
            # Check if it's address-of
            for child in parent.children:
                if child.type == '&' and child.text == b'&':
                    return AccessType.ADDRESS_OF
        
        # Update expressions (++, --, +=, etc)
        if parent.type == 'update_expression':
            return AccessType.READ_WRITE
        
        # Field expression - could be member access or method call
        if parent.type == 'field_expression':
            # Determine if we're the object or the field
            # In obj.field, obj is children[0], field is children[2] (after the dot)
            if parent.children[0] == node:
                # We're the object part - always READ (reading the object to access its member)
                return AccessType.READ
            else:
                # We're the field part - check what's happening to the field
                if grandparent and grandparent.type == 'call_expression':
                    # Check if the field_expression is being called
                    if grandparent.children[0] == parent:
                        # TODO: This records CALL access to just the method NAME, not the actual method entity
                        # For proper tracking, we need to:
                        # 1. Determine the type of the object (children[0])
                        # 2. Find the method entity in that class with matching signature
                        # 3. Record CALL access to that specific method entity
                        # This requires type resolution - see file header TODO for libclang integration
                        return AccessType.CALL  # The field is a method being called
                # Check if grandparent is assignment
                if grandparent and grandparent.type == 'assignment_expression':
                    # Check if we're on left side of assignment
                    if grandparent.children[0] == parent:
                        return AccessType.WRITE  # The field is being written to
                return AccessType.READ  # The field is being read
        
        # Direct call (function name)
        if parent.type == 'call_expression':
            # Check if we're the function being called
            if parent.children[0] == node:
                return AccessType.CALL
            return AccessType.READ  # We're an argument
        
        # Assignment expression
        if parent.type == 'assignment_expression':
            # Check which side we're on
            left_side = parent.children[0]
            if left_side == node:
                return AccessType.WRITE
            # Skip the operator, check if we're on right side
            for i, child in enumerate(parent.children):
                if child == node and i > 0:
                    return AccessType.READ
        
        # Argument list - parameters are being read
        if parent.type == 'argument_list':
            return AccessType.READ
        
        # Binary expressions - operands are read
        if parent.type == 'binary_expression':
            return AccessType.READ
        
        # Subscript expression - array/container access
        if parent.type == 'subscript_expression':
            # The array/container is being read
            if parent.children[0] == node:
                return AccessType.READ
            # The index is also being read
            return AccessType.READ
        
        # Qualified identifier - like Class::member
        if parent.type == 'qualified_identifier':
            # Check what the qualified identifier is used for
            grandparent = node.get_ancestor(2)
            if grandparent:
                if grandparent.type == 'declaration':
                    # If it's a qualified name in a declaration (e.g., Context::mDisplayModes),
                    # it's implementing/instantiating something already declared
                    if '::' in parent.text.decode():
                        return AccessType.IMPL
                    return AccessType.DEF
                elif grandparent.type == 'assignment_expression':
                    if grandparent.children[0] == parent:
                        return AccessType.WRITE
                    return AccessType.READ
            return AccessType.READ
        
        # Return statement - value is being read
        if parent.type == 'return_statement':
            return AccessType.READ
        
        # Default
        return AccessType.READ
    
    def _is_endpoint(self, node: StackTreeNode) -> bool:
        """
        Check if this node is an endpoint (should be tracked) 
        or just part of navigation path.
        
        For now, let's track everything and refine later.
        """
        # TODO: Implement proper endpoint detection once parent tracking is verified
        return True
    
    def _is_call_target(self, node: StackTreeNode) -> bool:
        """Check if this node is the target of a call expression"""
        if parent and parent.type == 'call_expression':
            # The first child is the function being called
            return parent.children and node == parent.children[0]
        return False
    
    def _is_terminal_field(self, node: StackTreeNode) -> bool:
        """Check if this field_identifier is the terminal field in a chain"""
        # TODO: Implement proper terminal field detection
        return True
    
    def _record_access(self, node: StackTreeNode, access_type: AccessType):
        """Record a detected access"""
        # Get identifier text
        identifier = node.text.decode() if node.text else ""
        
        # Get line and column
        trimmed_line = node.start_point[0] + 1  # tree-sitter uses 0-based
        column = node.start_point[1]
        
        # Map to original line
        original_line = self.line_mapping.get(trimmed_line, trimmed_line)
        
        # Get context snippet
        snippet = self._get_context_snippet(node)
        
        # Create access record
        access = Access(
            raw_identifier=identifier,
            access_type=access_type,
            file_path=self.current_file,
            trimmed_line=trimmed_line,
            original_line=original_line,
            column=column,
            context_function=self.current_function,
            context_class=self.current_class,
            context_namespace=self.current_namespace,
            context_snippet=snippet
        )
        
        self.accesses.append(access)
    
    def _is_declaration_context(self, node: StackTreeNode) -> bool:
        """
        Check if this node is part of a declaration context where
        identifiers should not be tracked as accesses.
        """
        # These node types are declarations, not accesses
        # Note: We only want the declaration part, not the body/initializer
        declaration_types = {
            'declaration',
            'field_declaration',
            'field_declaration_list',  # Class/struct member declarations
            'parameter_declaration',
            # Don't include function_declaration - we want to track accesses in function bodies
            # Don't include class_specifier - we want to track accesses in class bodies
            # Only pure type declarations:
            'type_descriptor',
            'abstract_declarator',
            'abstract_pointer_declarator',
            'abstract_array_declarator',
            'abstract_function_declarator',
            'abstract_reference_declarator'
        }
        
        # Check if this node itself is a declaration
        return node.type in declaration_types
    
    def _is_being_declared(self, node: StackTreeNode) -> bool:
        """
        Check if this specific identifier is the thing being declared,
        not just something used in a declaration's initializer.
        """
        parent = node.parent
        if not parent:
            return False
            
        # In an init_declarator, the first child is the identifier being declared
        if parent.type == 'init_declarator':
            return parent.children and node == parent.children[0]
            
        # If we are a field_identifier in a field_declaration
        if node.type == 'field_identifier':
            # Check if parent is field_declaration
            if parent and parent.type == 'field_declaration':
                return True
                
        # In a parameter_declaration, the identifier is being declared
        if parent.type == 'parameter_declaration':
            return True
            
        # In a function_declarator, the identifier is the function name being declared
        if parent.type == 'function_declarator':
            # Check if we're the function name (usually first identifier child)
            for child in parent.children:
                if child.type in ['identifier', 'field_identifier', 'qualified_identifier']:
                    return node == child
                    
        return False
    
    def _update_context(self, node: StackTreeNode):
        """Update namespace/class/function context based on node type"""
        if node.type == 'namespace_definition':
            # Extract namespace name
            for child in node.children:
                if child.type == 'namespace_identifier':
                    self.current_namespace = child.text.decode()
                    break
                    
        elif node.type in ['class_specifier', 'struct_specifier']:
            # Extract class/struct name
            for child in node.children:
                if child.type == 'type_identifier':
                    self.current_class = child.text.decode()
                    break
                    
        elif node.type == 'function_definition':
            # Extract function name from declarator
            for child in node.children:
                if child.type == 'function_declarator':
                    # Get the identifier from the declarator
                    for subchild in child.children:
                        if subchild.type in ['identifier', 'field_identifier', 'qualified_identifier']:
                            full_name = subchild.text.decode()
                            self.current_function = full_name
                            
                            # If it's a qualified identifier (Class::method), extract class context
                            if subchild.type == 'qualified_identifier' and '::' in full_name:
                                parts = full_name.rsplit('::', 1)
                                if len(parts) == 2:
                                    self.current_class = parts[0]
                                    self.current_function = parts[1]
                            break
                    break
                elif child.type == 'reference_declarator':
                    # Sometimes the function_declarator is inside a reference_declarator
                    for subchild in child.children:
                        if subchild.type == 'function_declarator':
                            for subsubchild in subchild.children:
                                if subsubchild.type in ['identifier', 'field_identifier', 'qualified_identifier']:
                                    full_name = subsubchild.text.decode()
                                    self.current_function = full_name
                                    
                                    # If it's a qualified identifier (Class::method), extract class context
                                    if subsubchild.type == 'qualified_identifier' and '::' in full_name:
                                        parts = full_name.rsplit('::', 1)
                                        if len(parts) == 2:
                                            self.current_class = parts[0]
                                            self.current_function = parts[1]
                                    break
                            break
    
    
    def _get_operator_text(self, node: StackTreeNode) -> str:
        """Extract operator text from a node"""
        # Find the operator child
        for child in node.children:
            if child.type in ['&', '*', '++', '--', '+', '-', '/', '%', '<<', '>>', 
                              '==', '!=', '<', '>', '<=', '>=', '&&', '||', '!']:
                return child.type
        
        # Fallback to extracting from text
        if node.text:
            text = node.text.decode()
            # Extract operator from text...
            # This is simplified - real implementation would be more robust
            return text
        
        return ""
    
    def _is_assignment_operator(self, node: StackTreeNode) -> bool:
        """Check if node is an assignment operator"""
        return node.type in ['=', '+=', '-=', '*=', '/=', '%=', '&=', '|=', '^=', '<<=', '>>=']
    
    def _get_context_snippet(self, node: StackTreeNode, context_lines: int = 1) -> str:
        """Get a code snippet around the access for context"""
        if not self.source_bytes:
            return ""
        
        # Get the line containing the access
        lines = self.source_bytes.decode().split('\n')
        line_idx = node.start_point[0]
        
        # Get surrounding lines
        start_idx = max(0, line_idx - context_lines)
        end_idx = min(len(lines), line_idx + context_lines + 1)
        
        snippet_lines = lines[start_idx:end_idx]
        return '\n'.join(snippet_lines)
    
    def resolve_accesses(self, accesses: List[Access]) -> List[Access]:
        """
        Resolve raw identifiers to database entities.
        This runs after all files have been analyzed.
        """
        with self.db.connect() as conn:
            cursor = conn.cursor()
            
            for access in accesses:
                entity_id, member_id = self._resolve_identifier(
                    cursor,
                    access.raw_identifier,
                    access.context_class,
                    access.context_namespace
                )
                
                access.entity_id = entity_id
                access.member_id = member_id
        
        return accesses
    
    def _resolve_identifier(self, cursor, identifier: str, 
                           context_class: Optional[str], 
                           context_namespace: Optional[str]) -> Tuple[Optional[int], Optional[int]]:
        """
        Resolve identifier to entity/member IDs.
        
        Resolution order:
        1. If contains ::, treat as qualified name
        2. Check as member of context class (if in class context)
        3. Check local namespace
        4. Check global namespace
        """
        # This is a simplified version - full implementation would be more sophisticated
        
        # Try to find as a member of the context class (NO PREFIX ASSUMPTIONS)
        if context_class:
            # Look for member in context class
            result = cursor.execute("""
                SELECT em.id, em.entity_id
                FROM entity_members em
                JOIN entities e ON em.entity_id = e.id
                WHERE e.short_name = ? AND em.name = ?
            """, (context_class, identifier)).fetchone()
            
            if result:
                return result['entity_id'], result['id']
        
        # Try to find as a function or type
        if '::' in identifier:
            # Qualified name
            result = cursor.execute("""
                SELECT id FROM entities WHERE canonical_name = ?
            """, (identifier,)).fetchone()
        else:
            # Unqualified - try with namespace context
            if context_namespace:
                full_name = f"{context_namespace}::{identifier}"
                result = cursor.execute("""
                    SELECT id FROM entities WHERE canonical_name = ?
                """, (full_name,)).fetchone()
            else:
                result = cursor.execute("""
                    SELECT id FROM entities WHERE short_name = ?
                """, (identifier,)).fetchone()
        
        if result:
            return result['id'], None
        
        return None, None

# Module exports
__all__ = ['StackBasedAccessAnalyzer', 'AccessType', 'Access', 'ContextFrame']


#!/usr/bin/env python3
"""
Search for C++ entities in Orkid database.
Simplified wrapper for Orkid-specific searching.
"""

import sys
import argparse
from pathlib import Path

# Add OBT to path
sys.path.append(str(Path(__file__).parent.parent.parent))
from obt.path import stage
from obt.deco import Deco
from obt.cpp_database_v2 import CppDatabaseV2
from obt.cpp_search_v2 import search_database

deco = Deco()

def create_parser():
    """Create argument parser for Orkid search"""
    parser = argparse.ArgumentParser(
        description='Search for C++ entities in Orkid database',
        formatter_class=argparse.RawDescriptionHelpFormatter
    )
    
    # Search pattern (optional)
    parser.add_argument(
        'pattern',
        nargs='?',
        help='Pattern to search for in entity names'
    )
    
    # Filtering options
    parser.add_argument(
        '-t', '--types', '--type',
        help='Entity types to search (comma-separated: class,struct,enum,function,typedef,alias,namespace)'
    )
    
    parser.add_argument(
        '-n', '--namespace',
        help='Filter by namespace'
    )
    
    parser.add_argument(
        '--exact',
        action='store_true',
        help='Exact name match instead of pattern match'
    )
    
    parser.add_argument(
        '-i', '--case-insensitive',
        action='store_true',
        help='Case-insensitive pattern matching'
    )
    
    parser.add_argument(
        '--templates-only',
        action='store_true',
        help='Show only template entities'
    )
    
    parser.add_argument(
        '--limit',
        type=int,
        default=0,
        help='Limit number of results (0 = unlimited)'
    )
    
    parser.add_argument(
        '--json',
        action='store_true',
        help='Output results as JSON'
    )
    
    parser.add_argument(
        '-v', '--verbose',
        action='store_true',
        help='Verbose output'
    )
    
    return parser

def format_namespace_json(results):
    """Format namespace results as JSON"""
    json_results = []
    for entity in results:
        json_obj = {
            'name': entity.short_name,
            'canonical_name': entity.canonical_name,
            'type': 'namespace',
            'namespace': None,
            'is_template': False,
            'template_params': None,
            'locations': []
        }
        json_results.append(json_obj)
    return json_results

def display_namespaces(results):
    """Display namespace results using column formatter"""
    if not results:
        print("// No matching namespaces found")
        return
    
    # Print header
    print("/////////////////////////////////////////////////////////////")
    print(f"// Found {len(results)} namespace{'s' if len(results) != 1 else ''}")
    print("/////////")
    print()
    
    # Column configuration
    LINE_LENGTH = 148
    depth_col_width = 8
    type_col_width = 12
    namespace_col_width = LINE_LENGTH - (depth_col_width + type_col_width)
    
    # Print header with reverse video
    header_configs = [
        (depth_col_width, 'left'),
        (type_col_width, 'left'),
        (namespace_col_width, 'left')
    ]
    header_texts = [
        'Depth',
        'Type',
        'Namespace'
    ]
    
    # Format the header
    header_line = deco.formatColumns(header_configs, header_texts)
    print(deco.reverseVideo(header_line.ljust(LINE_LENGTH)))
    
    # Sort namespaces by canonical name for hierarchical display
    sorted_results = sorted(results, key=lambda x: x.canonical_name)
    
    for entity in sorted_results:
        # Calculate namespace depth
        depth = entity.canonical_name.count('::')
        
        # Format columns
        depth_col = deco.white(str(depth).ljust(depth_col_width))
        type_col = deco.green('namespace'.ljust(type_col_width))
        
        # Add indentation based on depth for visual hierarchy
        indent = '  ' * min(depth, 5)  # Cap indentation at 5 levels
        namespace_col = deco.inf(indent + entity.canonical_name)
        
        # Use formatColumns for aligned output
        column_configs = [
            (depth_col_width, 'left'),
            (type_col_width, 'left'),
            (namespace_col_width, 'left')
        ]
        column_texts = [depth_col, type_col, namespace_col]
        output = deco.formatColumns(column_configs, column_texts)
        
        print(output)

def search_namespaces(db, pattern=None, limit=0):
    """Search for namespaces in the database"""
    with db.connect() as conn:
        cursor = conn.cursor()
        
        # Build query for distinct namespaces
        if pattern:
            # Support wildcards
            if '*' in pattern:
                pattern = pattern.replace('*', '%')
                query = """
                    SELECT DISTINCT namespace 
                    FROM entities 
                    WHERE namespace LIKE ? 
                    AND namespace IS NOT NULL 
                    ORDER BY namespace
                """
            else:
                query = """
                    SELECT DISTINCT namespace 
                    FROM entities 
                    WHERE namespace LIKE '%' || ? || '%'
                    AND namespace IS NOT NULL 
                    ORDER BY namespace
                """
            params = (pattern,)
        else:
            query = """
                SELECT DISTINCT namespace 
                FROM entities 
                WHERE namespace IS NOT NULL 
                ORDER BY namespace
            """
            params = ()
        
        if limit > 0:
            query += f" LIMIT {limit}"
            
        cursor.execute(query, params)
        
        # Create pseudo-entities for namespaces
        results = []
        for row in cursor.fetchall():
            namespace = row[0]
            if namespace:  # Skip empty namespaces
                # Create a simple namespace entity
                from types import SimpleNamespace
                entity = SimpleNamespace()
                entity.name = namespace
                entity.short_name = namespace.split('::')[-1] if '::' in namespace else namespace
                entity.canonical_name = namespace
                entity.entity_type = 'namespace'
                entity.namespace = None  # Namespaces don't have parent namespaces in our model
                entity.is_template = False
                entity.template_params = None
                entity.locations = []
                entity.file_path = ''
                entity.line_number = 0
                results.append(entity)
        
        return results

def main():
    """Main entry point"""
    parser = create_parser()
    args = parser.parse_args()
    
    # Check if Orkid database exists
    db_path = stage() / "cpp_db_v2_orkid.db"
    if not db_path.exists():
        print(f"{deco.red('Orkid database not found!')}")
        print(f"Expected at: {db_path}")
        print(f"Run 'ork.cpp.db.build.py -m <modules>' to build it first")
        sys.exit(1)
    
    # Open database and search
    try:
        db = CppDatabaseV2(db_path)
        
        # Check if searching for namespaces specifically
        is_namespace_search = args.types and 'namespace' in args.types.lower()
        
        if is_namespace_search:
            # Search for namespaces
            results = search_namespaces(db, args.pattern, args.limit)
        else:
            # Add database to args for normal search function
            args.db = str(db_path)
            
            # Perform normal search
            results = search_database(db, args)
        
        if args.json:
            import json
            if is_namespace_search:
                # Use custom formatter for namespaces
                json_results = format_namespace_json(results)
            else:
                # Import JSON functionality from search_v2
                from obt.cpp_search_v2 import format_json_results
                json_results = format_json_results(results)
            print(json.dumps(json_results, indent=2))
        else:
            if is_namespace_search:
                # Custom display for namespaces
                display_namespaces(results)
            else:
                # Use display functionality
                from obt.cpp_display_v2 import CppEntityDisplayV2
                display = CppEntityDisplayV2()
                display.display_entities(results)
            
    except Exception as e:
        print(f"{deco.red(f'Error: {e}')}")
        sys.exit(1)

if __name__ == '__main__':
    main()


from obt import pathtools
from obt.buildtrace import buildTrace
import shutil, os

###############################################################################

def _copy_over(what, where):
  #os.chmod(where, 777) #?? still can raise exception
  shutil.copy(what, where)

###############################################################################

def rmdir(p,force=False):
  #####################################
  class _X:
    def __init__(self):
      pass
    def __call__(self):
      print({"l_rmdir":"src_dir(%s)"%str(p)})
      pathtools.rmdir(p,force=force)
  #####################################
  return _X()

###############################################################################

def mkdir(p,
          clean=False,
          parents=False):
  #####################################
  class _X:
    def __init__(self):
      pass
    def __call__(self):
      print({"l_mkdir":"src_dir(%s)"%str(p)})
      pathtools.mkdir(p,
                      clean=clean,
                      parents=parents)
  #####################################
  return _X()

###############################################################################

def copy_files(src_dir=None,
               dst_dir=None,
               pattern=None,
               modeset=""):
  class _X:
    def __init__(self):
      pass
    def __call__(self):
      print({"l_copyfiles":"src_dir(%s)"%str(src_dir)})
      print(src_dir)            
      print(pattern)            
      print(dst_dir)            
      print(modeset)            
      matched = pathtools.patglob(src_dir,pattern)
      for src_item in matched:
        s_path = os.path.split(src_item)
        dest_path = dst_dir/s_path[1]
        print("copy %s to %s"%(src_item,dest_path))
        _copy_over(src_item,dest_path)
  return _X()



#!/usr/bin/env python3

import argparse
import os
from pathlib import Path
from obt import systemd
from obt.deco import Deco

def main():
    parser = argparse.ArgumentParser(description="Generate a standalone shell script to run a systemd service")
    parser.add_argument("service_name", help="Service module name (e.g., ork_logger_http)")
    parser.add_argument("-o", "--output", required=True, help="Output shell script path")
    parser.add_argument("--user", action="store_true", help="Use user service file (~/.config/systemd/user/)")
    parser.add_argument("--system", action="store_true", help="Use system service file (/etc/systemd/system/)")
    parser.add_argument("--from", dest="from_file", help="Read from specific service file path")

    args = parser.parse_args()
    deco = Deco()

    try:
        # Determine service file location
        if args.from_file:
            service_file = Path(args.from_file).expanduser()
        else:
            # Verify service module exists
            svc = systemd.requires(args.service_name)
            info = svc.info()

            # First try to find in staging folder (most up-to-date)
            obt_stage = os.environ.get("OBT_STAGE")
            if obt_stage:
                staging_service_file = Path(obt_stage) / "systemd" / args.service_name / f"{args.service_name}.service"
                if staging_service_file.exists():
                    service_file = staging_service_file
                elif args.system:
                    service_file = Path(f"/etc/systemd/system/{args.service_name}.service")
                elif args.user:
                    service_file = Path.home() / ".config" / "systemd" / "user" / f"{args.service_name}.service"
                else:
                    # Auto-detect based on module config
                    if info.get('as_system_service', False):
                        service_file = Path(f"/etc/systemd/system/{args.service_name}.service")
                    else:
                        service_file = Path.home() / ".config" / "systemd" / "user" / f"{args.service_name}.service"
            else:
                if args.system:
                    service_file = Path(f"/etc/systemd/system/{args.service_name}.service")
                elif args.user:
                    service_file = Path.home() / ".config" / "systemd" / "user" / f"{args.service_name}.service"
                else:
                    # Auto-detect based on module config
                    if info.get('as_system_service', False):
                        service_file = Path(f"/etc/systemd/system/{args.service_name}.service")
                    else:
                        service_file = Path.home() / ".config" / "systemd" / "user" / f"{args.service_name}.service"

        if not service_file.exists():
            print(deco.err(f"Service file not found: {service_file}"))
            print(deco.inf("Run 'obt.systemd.create.py' first"))
            exit(1)

        # Parse service file
        env_vars = {}
        exec_start = None

        with open(service_file) as f:
            for line in f:
                line = line.strip()
                # Parse Environment= lines
                if line.startswith('Environment='):
                    # Extract: Environment="VAR=value"
                    env_str = line[len('Environment='):].strip()
                    # Remove quotes
                    if env_str.startswith('"') and env_str.endswith('"'):
                        env_str = env_str[1:-1]
                    # Split on first =
                    if '=' in env_str:
                        var_name, var_value = env_str.split('=', 1)
                        env_vars[var_name] = var_value
                # Parse ExecStart= line
                elif line.startswith('ExecStart='):
                    exec_start = line[len('ExecStart='):].strip()

        if not exec_start:
            print(deco.err("No ExecStart found in service file"))
            exit(1)

        # Generate shell script
        output_path = Path(args.output).expanduser()

        with open(output_path, 'w') as f:
            f.write("#!/bin/bash\n")
            f.write(f"# Generated runner script for {args.service_name} service\n")
            f.write(f"# Source service file: {service_file}\n")
            f.write("\n")

            # Export environment variables
            for var_name, var_value in env_vars.items():
                # Escape single quotes in value
                escaped_value = var_value.replace("'", "'\\''")
                f.write(f"export {var_name}='{escaped_value}'\n")

            f.write("\n")
            f.write("# Execute service command\n")
            f.write(f"{exec_start}\n")

        # Make executable
        output_path.chmod(0o755)

        print(deco.key(f"Created runner script: {output_path}"))
        print(deco.inf(f"Service: {args.service_name}"))
        print(deco.inf(f"Exported {len(env_vars)} environment variables"))
        print(deco.val(f"Run with: {output_path}"))

    except RuntimeError as e:
        print(deco.err(str(e)))
        exit(1)

if __name__ == "__main__":
    main()



import os, shutil
import obt.deco
from obt import path, dep
from obt.command import Command

def _clean_build_dir(name_or_list):
  """Remove build/ directory for local path installs to avoid stale artifacts."""
  items = [name_or_list] if isinstance(name_or_list, str) else name_or_list
  for item in items:
    p = os.path.abspath(item)
    if os.path.isdir(p):
      build_dir = os.path.join(p, "build")
      if os.path.isdir(build_dir):
        shutil.rmtree(build_dir)

def _command(name_or_list,cmd):
  #Command().exec()
  #pip_executable = path.stage()/"bin"/"pip3"
  PYTHON = dep.instance("python")
  python_executable = PYTHON.executable
  cmd_prefix = [python_executable,"-m","pip",cmd]

  if cmd == "install":
    _clean_build_dir(name_or_list)

  rval = 0
  if(isinstance(name_or_list,str)):
    r = Command(cmd_prefix+[name_or_list]).exec()
    rval = r
  elif (isinstance(name_or_list,list)):
    r = Command(cmd_prefix+name_or_list).exec()
    rval = r

  return rval

def install(name_or_list):
  return _command(name_or_list,"install")

def uninstall(name_or_list):
  return _command(name_or_list,"uninstall")



from yarl import URL
from obt import dep, path, pathtools, command

###############################################################################

class dspstretch(dep.StdProvider):
  VERSION = "c315378"
  name = "dspstretch"
  def __init__(self):
    build_dir = path.builds()/dspstretch.name
    super().__init__(dspstretch.name)
    
    self.dest_header_path = path.includes()/"dspstretch"
    self._builder = self.createBuilder(dep.CustomBuilder)
    self._builder._cleanOnClean = False

    def before_build():
      print("before_build")
      pathtools.ensureDirectoryExists(self.dest_header_path)
      items = pathtools.patglob(self.source_root, "*.h")
      copy_commands = []
      for item in items:
        print(item)
        cmd = command.Command([
          "cp",str(item), str(self.dest_header_path)+"/"
          ])
        copy_commands.append(cmd)
      self._builder._cleanbuildcommands = copy_commands
      self._builder._incrbuildcommands = copy_commands
      print("copy_commands: %s" % copy_commands)
    
    self._builder._invokeBeforeBuild = before_build

  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=dspstretch.name,
                             repospec="Signalsmith-Audio/signalsmith-stretch",
                             revision=dspstretch.VERSION,
                             recursive=True)

  #######################################################################

  def areRequiredSourceFilesPresent(self):
    print(self.source_root)
    return (self.source_root/"README.md").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.includes()/"dspstretch"/"signalsmith-stretch.h").exists()



#!/usr/bin/env python3

from obt import systemd
from obt.deco import Deco

def main():
    deco = Deco()
    services = systemd.enumerate()

    if not services:
        print("No systemd service modules found.")
        return

    print(deco.key("Available Systemd Services:"))
    for service_name in services:
        print(f"  {deco.val(service_name)}")

if __name__ == "__main__":
    main()



###############################################################################
# obt._deploy_fixup
#
# Launch-time fixup for relocated deployments.
#
# A deployment built by ork.deploy.macos.relocatable.py contains text files
# with absolute paths baked to the original target directory. If the deploy
# is moved to a new location, those paths break.
#
# This module detects relocation and patches text files so the deployment
# works from any directory. It is designed to be called from any launcher
# script (shell launcher, GUI app, etc.) before the main OBT environment
# is initialized.
#
# Usage from a launcher:
#   from obt._deploy_fixup import fixup_if_relocated
#   fixup_if_relocated("/path/to/deploy")
#
# The fixup is idempotent and fast on subsequent launches (skipped when the
# marker file matches the current location).
###############################################################################

import os
import sys

# Marker file written at the deploy root to record the "known good" path.
_MARKER_FILENAME = ".deploy_path"

###############################################################################
# Public API
###############################################################################

def fixup_if_relocated(deploy_root):
  """Check if the deployment has been moved and fix paths if so.

  This is the main entry point. Call it from any launcher before
  initializing the OBT environment.

  Args:
    deploy_root: Absolute path to the deployment root directory.

  Returns:
    True if a fixup was performed, False if no fixup was needed.
  """
  deploy_root = os.path.realpath(deploy_root)
  marker_path = os.path.join(deploy_root, _MARKER_FILENAME)

  old_root = _read_marker(marker_path)
  if old_root == deploy_root:
    return False  # No relocation — fast path

  if old_root is None:
    # First run or marker missing — write marker, no fixup needed
    # (paths should already match from the build)
    _write_marker(marker_path, deploy_root)
    return False

  # Deployment was moved: old_root → deploy_root
  print(f"[deploy-fixup] Relocation detected:")
  print(f"  old: {old_root}")
  print(f"  new: {deploy_root}")

  count = _apply_fixup(deploy_root, old_root)

  print(f"[deploy-fixup] Fixed {count} files.")
  _write_marker(marker_path, deploy_root)
  return True

###############################################################################
# Fixup implementation
###############################################################################

# Directories (relative to deploy root) containing text files that need fixup.
# Each entry is (rel_dir, file_filter) where file_filter is a callable.
_FIXUP_TARGETS = [
  # Venv configs — critical for Python to find its stdlib
  ("obt_venv", lambda f: f == "pyvenv.cfg"),
  ("pyvenv",   lambda f: f == "pyvenv.cfg"),

  # Venv bin scripts (pip, activate, etc.)
  ("obt_venv/bin", lambda f: not f.endswith(('.pyc',))),
  ("pyvenv/bin",   lambda f: not f.endswith(('.pyc',))),

  # pkg-config files
  ("lib/pkgconfig",   lambda f: f.endswith('.pc')),
  ("lib64/pkgconfig", lambda f: f.endswith('.pc')),

  # CMake config files
  ("lib/cmake", lambda f: f.endswith('.cmake')),
]

def _apply_fixup(deploy_root, old_root):
  """Replace old_root with deploy_root in all known text files.

  Args:
    deploy_root: New absolute path to the deployment.
    old_root: Previous absolute path embedded in text files.

  Returns:
    Number of files modified.
  """
  count = 0

  for rel_dir, file_filter in _FIXUP_TARGETS:
    abs_dir = os.path.join(deploy_root, rel_dir)
    if not os.path.isdir(abs_dir):
      continue
    for fname in os.listdir(abs_dir):
      if not file_filter(fname):
        continue
      fpath = os.path.join(abs_dir, fname)
      if not os.path.isfile(fpath) or os.path.islink(fpath):
        continue
      if _fix_file(fpath, old_root, deploy_root):
        count += 1

  return count


def _fix_file(fpath, old_str, new_str):
  """Replace old_str with new_str in a single text file.

  Returns True if the file was modified.
  """
  try:
    with open(fpath, 'r', errors='replace') as f:
      content = f.read()
  except (OSError, UnicodeDecodeError):
    return False

  if old_str not in content:
    return False

  content = content.replace(old_str, new_str)
  try:
    with open(fpath, 'w') as f:
      f.write(content)
  except OSError:
    return False

  return True

###############################################################################
# Marker file
###############################################################################

def _read_marker(marker_path):
  """Read the deploy path marker. Returns None if absent or unreadable."""
  try:
    with open(marker_path, 'r') as f:
      return f.read().strip()
  except (OSError, ValueError):
    return None


def _write_marker(marker_path, deploy_root):
  """Write the deploy path marker."""
  try:
    with open(marker_path, 'w') as f:
      f.write(deploy_root + '\n')
  except OSError:
    pass  # Non-fatal — fixup will just re-run next time



#!/usr/bin/env python3
"""
Test script for parallel ingestion and parsing
"""

import sys
import argparse
import time
import logging
from pathlib import Path
from datetime import datetime
from concurrent.futures import ProcessPoolExecutor, as_completed

from obt import cpp_ingest
from obt import cpp_database_v2
from obt import ork_cppdb
from obt import cpp_parser_descent
from obt import host

# Configure logging
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s.%(msecs)03d - %(levelname)s - %(message)s',
    datefmt='%H:%M:%S'
)
logger = logging.getLogger(__name__)

def parse_single_file(args):
    """
    Parse a single file from the database
    Returns tuple of (file_path, parse_time, entity_count, error)
    """
    file_path, db_path = args
    start_time = time.perf_counter()
    
    try:
        # Connect to database
        db = cpp_database_v2.CppDatabaseV2(db_path)
        with db.connect() as conn:
            # Get trimmed source
            cursor = conn.execute("""
                SELECT trimmed_source FROM source_files 
                WHERE file_path = ?
            """, (file_path,))
            row = cursor.fetchone()
            
            if not row:
                return file_path, 0, 0, f"File not found in database"
            
            trimmed_source = row[0]
            
        # Parse with recursive descent parser
        parser = cpp_parser_descent.CppParser()
        entities = parser.parse_source(trimmed_source.encode('utf-8'), file_path)
        
        parse_time = time.perf_counter() - start_time
        return file_path, parse_time, len(entities), None
        
    except Exception as e:
        parse_time = time.perf_counter() - start_time
        return file_path, parse_time, 0, str(e)

def parallel_parse_phase(db_path: Path, max_workers: int = 4):
    """
    Parse all ingested files in parallel
    """
    logger.info("=" * 60)
    logger.info("Starting parallel parsing phase")
    
    phase_start = time.perf_counter()
    
    # Get list of files to parse
    db = cpp_database_v2.CppDatabaseV2(db_path)
    with db.connect() as conn:
        cursor = conn.execute("SELECT file_path FROM source_files")
        file_paths = [row[0] for row in cursor.fetchall()]
    
    logger.info(f"Found {len(file_paths)} files to parse")
    
    # Prepare arguments for parallel processing
    parse_args = [(fp, db_path) for fp in file_paths]
    
    # Parse in parallel
    results = {
        'success': [],
        'failed': [],
        'total_entities': 0,
        'total_parse_time': 0
    }
    
    completed = 0
    total = len(file_paths)
    
    with ProcessPoolExecutor(max_workers=max_workers) as executor:
        futures = {
            executor.submit(parse_single_file, args): args[0]
            for args in parse_args
        }
        
        for future in as_completed(futures):
            file_path = futures[future]
            try:
                fp, parse_time, entity_count, error = future.result(timeout=30)
                
                if error:
                    results['failed'].append({
                        'path': fp,
                        'error': error,
                        'time': parse_time
                    })
                else:
                    results['success'].append({
                        'path': fp,
                        'entities': entity_count,
                        'time': parse_time
                    })
                    results['total_entities'] += entity_count
                    results['total_parse_time'] += parse_time
                    
            except Exception as e:
                results['failed'].append({
                    'path': file_path,
                    'error': str(e),
                    'time': 0
                })
            
            completed += 1
            if completed % 10 == 0 or completed == total:
                percent = (completed / total) * 100
                elapsed = time.perf_counter() - phase_start
                rate = completed / elapsed if elapsed > 0 else 0
                logger.info(f"  Parse progress: {completed}/{total} files ({percent:.1f}%) - {rate:.1f} files/sec")
    
    phase_time = time.perf_counter() - phase_start
    
    # Report results
    logger.info("=" * 60)
    logger.info("Parsing Phase Statistics:")
    logger.info(f"  Files parsed: {len(results['success'])}")
    logger.info(f"  Files failed: {len(results['failed'])}")
    logger.info(f"  Total entities found: {results['total_entities']:,}")
    
    if results['success']:
        avg_parse_time = results['total_parse_time'] / len(results['success'])
        avg_entities = results['total_entities'] / len(results['success'])
        logger.info(f"  Average parse time: {avg_parse_time:.3f}s per file")
        logger.info(f"  Average entities per file: {avg_entities:.1f}")
        logger.info(f"  Total CPU time: {results['total_parse_time']:.1f}s")
        logger.info(f"  Wall time: {phase_time:.1f}s")
        logger.info(f"  Speedup: {results['total_parse_time']/phase_time:.1f}x")
    
    # Show some failed files if any
    if results['failed']:
        logger.warning(f"Failed to parse {len(results['failed'])} files:")
        for item in results['failed'][:5]:
            logger.warning(f"  {item['path']}: {item['error']}")
        if len(results['failed']) > 5:
            logger.warning(f"  ... and {len(results['failed']) - 5} more")
    
    return results

def test_module_ingestion(module_name):
    """Test ingesting files from a module"""
    
    # Start total timer
    total_start = time.perf_counter()
    
    # Create test database
    db_path = Path.home() / '.staging-aug18' / 'cpp_db_v2_test_ingest.db'
    
    # Get files to test
    logger.info(f"Testing ingestion of module: {module_name}")
    logger.info(f"Database path: {db_path}")
    
    file_discovery_start = time.perf_counter()
    source_paths = ork_cppdb.get_orkid_paths(modules=[module_name])
    files = ork_cppdb.find_source_files(source_paths)
    file_discovery_time = time.perf_counter() - file_discovery_start
    
    logger.info(f"Found {len(files)} files in module {module_name} ({file_discovery_time:.3f}s)")
    
    # Set up include paths for Orkid
    include_paths = ork_cppdb.get_orkid_include_paths()
    logger.info(f"Using {len(include_paths)} include paths")
    
    # Create ingestor with Orkid settings
    ingestor = cpp_ingest.CppIngestor(
        defines={'ORKID_INGEST_TEST': '1'},
        include_paths=include_paths,
        defines_preset='macos'
    )
    
    if not files:
        logger.warning(f"No files found for module {module_name}")
        return
    
    logger.info(f"Starting parallel ingestion of {len(files)} files...")
    
    # Use parallel ingestion
    ingestion_start = time.perf_counter()
    results = ingestor.ingest_files_parallel(files, max_workers=host.NumCores)
    ingestion_time = time.perf_counter() - ingestion_start
    
    # Calculate statistics
    total_raw = sum(item['raw_size'] for item in results['success'])
    total_preprocessed = sum(item['preprocessed_size'] for item in results['success'])
    total_trimmed = sum(item['trimmed_size'] for item in results['success'])
    
    # Performance metrics
    files_per_second = len(results['success']) / ingestion_time if ingestion_time > 0 else 0
    mb_per_second = (total_raw / (1024 * 1024)) / ingestion_time if ingestion_time > 0 else 0
    
    logger.info(f"Ingestion completed in {ingestion_time:.3f}s")
    logger.info(f"Performance: {files_per_second:.1f} files/sec, {mb_per_second:.2f} MB/sec")
    
    logger.info("=" * 60)
    logger.info("Ingestion Statistics:")
    logger.info(f"  Files processed: {len(results['success'])}")
    logger.info(f"  Files failed: {len(results['failed'])}")
    logger.info(f"  Total raw size: {total_raw:,} bytes ({total_raw/(1024*1024):.2f} MB)")
    logger.info(f"  Total preprocessed size: {total_preprocessed:,} bytes ({total_preprocessed/(1024*1024):.2f} MB)")
    logger.info(f"  Total trimmed size: {total_trimmed:,} bytes ({total_trimmed/(1024*1024):.2f} MB)")
    
    if total_raw > 0:
        logger.info(f"  Average expansion: {total_preprocessed / total_raw:.1f}x")
        logger.info(f"  Trimming saved: {(1 - total_trimmed / total_preprocessed) * 100:.2f}% of preprocessed")
        logger.info(f"  Trimming ratio: {total_trimmed / total_raw:.3f}x of original")
    
    # Per-file statistics
    if results['success']:
        preprocess_times = []
        trim_ratios = []
        for item in results['success']:
            if 'preprocess_time' in item:
                preprocess_times.append(item['preprocess_time'])
            if item['preprocessed_size'] > 0:
                trim_ratios.append(item['trimmed_size'] / item['preprocessed_size'])
        
        if preprocess_times:
            avg_preprocess = sum(preprocess_times) / len(preprocess_times)
            logger.info(f"  Avg preprocessing time: {avg_preprocess:.3f}s per file")
        
        if trim_ratios:
            avg_trim_ratio = sum(trim_ratios) / len(trim_ratios)
            logger.info(f"  Avg trim ratio: {(1 - avg_trim_ratio) * 100:.2f}% removed per file")
    
    # Store in database
    logger.info("=" * 60)
    logger.info(f"Storing in database: {db_path}")
    
    db_start = time.perf_counter()
    db = cpp_database_v2.CppDatabaseV2(db_path)
    
    with db.connect() as conn:
        # Ensure trimmed_source column exists
        cursor = conn.execute("PRAGMA table_info(source_files)")
        columns = [row[1] for row in cursor.fetchall()]
        if 'trimmed_source' not in columns:
            conn.execute("ALTER TABLE source_files ADD COLUMN trimmed_source TEXT")
        
        # Store each file
        stored_count = 0
        for item in results['success']:
            conn.execute("""
                INSERT OR REPLACE INTO source_files 
                (file_path, file_name, raw_source, preprocessed_source, trimmed_source, file_size)
                VALUES (?, ?, ?, ?, ?, ?)
            """, (
                str(item['path']),
                item['path'].name,
                item['raw_source'].decode('utf-8', errors='ignore'),
                item['preprocessed_source'].decode('utf-8', errors='ignore'),
                item['trimmed_source'].decode('utf-8', errors='ignore'),
                item['raw_size']
            ))
            stored_count += 1
        
        conn.commit()
    
    db_time = time.perf_counter() - db_start
    logger.info(f"Database storage completed in {db_time:.3f}s")
    logger.info(f"  Stored {stored_count} files ({stored_count/db_time:.1f} files/sec)")
    
    # Show failed files if any
    if results['failed']:
        logger.warning(f"Failed files ({len(results['failed'])} total):")
        for item in results['failed'][:5]:  # Show first 5
            logger.warning(f"  {item['path']}: {item['error']}")
        if len(results['failed']) > 5:
            logger.warning(f"  ... and {len(results['failed']) - 5} more")
    
    # Total time
    total_time = time.perf_counter() - total_start
    logger.info("=" * 60)
    logger.info("Overall Performance Summary:")
    logger.info(f"  Total time: {total_time:.3f}s")
    logger.info(f"  File discovery: {file_discovery_time:.3f}s ({file_discovery_time/total_time*100:.1f}%)")
    logger.info(f"  Ingestion: {ingestion_time:.3f}s ({ingestion_time/total_time*100:.1f}%)")
    logger.info(f"  Database storage: {db_time:.3f}s ({db_time/total_time*100:.1f}%)")
    logger.info(f"  Overall throughput: {len(results['success'])/total_time:.1f} files/sec")
    
    return db_path

def main():
    parser = argparse.ArgumentParser(description='Test C++ file ingestion and parsing')
    
    # Get available modules for help text
    available = ', '.join(ork_cppdb.list_available_modules())
    
    parser.add_argument('-m', '--module',
                       help=f'Module name to test. Available: {available}')
    parser.add_argument('--list-modules', action='store_true',
                       help='List available modules with descriptions')
    parser.add_argument('--parse', action='store_true',
                       help='Also run parallel parsing phase after ingestion')
    parser.add_argument('--parse-workers', type=int, default=host.NumCores,
                       help=f'Number of parallel workers for parsing (default: {host.NumCores})')
    
    args = parser.parse_args()
    
    if args.list_modules:
        print("Available modules:")
        for module in ork_cppdb.list_available_modules():
            desc = ork_cppdb.get_module_description(module)
            print(f"  {module:12} - {desc}")
        sys.exit(0)
    
    if not args.module:
        parser.error("Module name required (use -m MODULE)")
    
    # Run ingestion
    db_path = test_module_ingestion(args.module)
    
    # Run parsing if requested
    if args.parse and db_path:
        parallel_parse_phase(db_path, max_workers=args.parse_workers)

if __name__ == '__main__':
    main()


#!/usr/bin/env python3
###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, sys, pathlib, argparse, multiprocessing, json

as_main = (__name__ == '__main__')

###########################################

Path = pathlib.Path
curwd = Path(os.getcwd())
file_path = os.path.realpath(__file__)
file_dir = os.path.dirname(file_path)
sys.path.append(str(file_dir))

###########################################

parser = argparse.ArgumentParser(description='obt.build environment creator')
parser.add_argument('--stagedir', metavar="createdir", help='create staging folder and enter session' )
parser.add_argument('--project', action='append', metavar="PROJECTLIST", help='append project directory' )
parser.add_argument('--prompt', metavar="prompt", help='prompt suffix' )
parser.add_argument("--numcores", metavar="numcores", help="numcores for environment")
parser.add_argument("--quiet", action="store_true", help="no output")
parser.add_argument('--novars', action="store_true", help='do not set env vars' )
parser.add_argument('--obttrace',action="store_true",help='enable OBT buildtrace logging')

parser.add_argument('--wipe', action="store_true", help='wipe old staging folder' )
parser.add_argument('--sshkey',metavar="sshkey",help='ssh key to use with OBT/GIT')

args = vars(parser.parse_args())

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

###########################################
# parse args and generate config / core environment vars
###########################################

from _obt_config import configFromCommandLine, initializeDependencyEnvironments
obt_config = configFromCommandLine(args)
      
###########################################
# wipe old staging folder ?
###########################################

if obt_config.stage_dir.exists() and args["wipe"]==False:
  print("Not going to wipe your staging folder<%s> unless you ask... use --wipe"%obt_config.stage_dir)
  sys.exit(0)
if args["wipe"] and obt_config.stage_dir.exists():
  os.system( "rm -rf %s"%obt_config.stage_dir)

###########################################

if args["obttrace"]==True:
  import obt._globals as _glob
  _glob.enableBuildTracing()

##########################################

import obt._envutils 
envsetup = obt._envutils.EnvSetup(obt_config)

###########################################
# Create staging folder, scripts
###########################################

import obt.path

obt.path.prefix().mkdir(parents=True,exist_ok=False)
envsetup.lazyMakeDirs()
envsetup.genBashRc(obt_config,obt_config.stage_dir/".bashrc")
envsetup.genLaunchScript(out_path=obt_config.stage_dir/"obt-launch-env")

initializeDependencyEnvironments(envsetup)

###########################################
# build mandatory dependencies
###########################################

print(os.environ)
print(os.environ["OBT_MODULES_PATH"])
os.system("ls %s" % os.environ["OBT_MODULES_PATH"])

MANDATORY_DEPS = ["cmake","python","pydefaults","vulkan"]

import obt.dep
for item in MANDATORY_DEPS:
  dep = obt.dep.instance(item)
  dep.provide()



###############################################################################
# Orkid Build System
# Copyright 2010-2023, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from obt import dep, log, path, host, env, osrelease
from obt.command import Command
###############################################################################
NAME = "tbb"
VERSION = "v2022.0.0"
###############################################################################
class tbb(dep.StdProvider):
  def __init__(self):
    super().__init__(NAME)

    self._builder = self.createBuilder(dep.CMakeBuilder)
    self._builder._cmakeenv = {
      "BUILD_SHARED_LIBS": "ON",
      "TBB_TEST": "OFF",
      "TBB_STRICT": "OFF",
      "TBB_EXAMPLES": "OFF",
    }

    # On certain Ubuntu versions, use gcc-11
    if host.IsLinux:
      desc = osrelease.descriptor()
      use_gcc_11 = desc.version_id in ["23.10", "24.04"]
      if use_gcc_11:
        self._builder._cmakeenv["CMAKE_CXX_COMPILER"] = "g++-11"
        self._builder._cmakeenv["CMAKE_C_COMPILER"] = "gcc-11"

  def env_init(self):
    log.marker("registering TBB(%s) SDK" % VERSION)

  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=NAME,
                             repospec="uxlfoundation/oneTBB",
                             revision=VERSION,
                             recursive=False)

  ###############################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()

  def areRequiredBinaryFilesPresent(self):
    return path.decorate_obt_lib("tbb").exists()
###############################################################################




#!/usr/bin/env python3

import argparse
import os
import subprocess
from pathlib import Path
from obt import systemd
from obt.deco import Deco

def main():
    parser = argparse.ArgumentParser(description="Run a systemd service in foreground with its environment")
    parser.add_argument("service_name", help="Service module name (e.g., ork_logger_http)")
    parser.add_argument("--user", action="store_true", help="Use user service file (~/.config/systemd/user/)")
    parser.add_argument("--system", action="store_true", help="Use system service file (/etc/systemd/system/)")

    args = parser.parse_args()
    deco = Deco()

    try:
        # Verify service module exists
        svc = systemd.requires(args.service_name)
        info = svc.info()

        # Determine service file location
        if args.system:
            service_file = Path(f"/etc/systemd/system/{args.service_name}.service")
        elif args.user:
            service_file = Path.home() / ".config" / "systemd" / "user" / f"{args.service_name}.service"
        else:
            # Auto-detect based on module config
            if info.get('as_system_service', False):
                service_file = Path(f"/etc/systemd/system/{args.service_name}.service")
            else:
                service_file = Path.home() / ".config" / "systemd" / "user" / f"{args.service_name}.service"

        if not service_file.exists():
            print(deco.err(f"Service file not found: {service_file}"))
            print(deco.inf("Run 'obt.systemd.create.py --deploy' first"))
            exit(1)

        # Parse service file
        env_vars = {}
        exec_start = None

        with open(service_file) as f:
            for line in f:
                line = line.strip()
                # Parse Environment= lines
                if line.startswith('Environment='):
                    # Extract: Environment="VAR=value"
                    env_str = line[len('Environment='):].strip()
                    # Remove quotes
                    if env_str.startswith('"') and env_str.endswith('"'):
                        env_str = env_str[1:-1]
                    # Split on first =
                    if '=' in env_str:
                        var_name, var_value = env_str.split('=', 1)
                        env_vars[var_name] = var_value
                # Parse ExecStart= line
                elif line.startswith('ExecStart='):
                    exec_start = line[len('ExecStart='):].strip()

        if not exec_start:
            print(deco.err("No ExecStart found in service file"))
            exit(1)

        print(deco.key(f"Running service: {args.service_name}"))
        print(deco.inf(f"Service file: {service_file}"))
        print(deco.inf(f"Setting {len(env_vars)} environment variables"))
        print(deco.key(f"Command: {exec_start}"))
        print(deco.val("=" * 80))

        # Set environment and run command
        env = os.environ.copy()
        env.update(env_vars)

        # Run the command in foreground
        subprocess.run(exec_start, shell=True, env=env)

    except RuntimeError as e:
        print(deco.err(str(e)))
        exit(1)
    except KeyboardInterrupt:
        print(deco.inf("\nInterrupted"))
        exit(0)

if __name__ == "__main__":
    main()



###############################################################################
# Orkid Build System
# Copyright 2010-2024, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from obt import dep, path
###############################################################################
class oscar64(dep.StdProvider):
  name = "oscar64"
  VERSION = "v1.32.268"
  def __init__(self):
    super().__init__(oscar64.name)
    self._builder = dep.CMakeBuilder(oscar64.name)
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=oscar64.name,
                             repospec="drmortalwombat/oscar64",
                             revision=oscar64.VERSION,
                             recursive=False,
                             disable_tarball=True)
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.bin()/"oscar64").exists()



#!/usr/bin/env python3
"""
Display members of C++ classes/structs.
Part of the OBT C++ analysis toolkit.
"""

import sys
from pathlib import Path
sys.path.append(str(Path(__file__).parent.parent.parent))

from obt.cpp_command_base import CppCommandBase
from obt.cpp_database_v2 import CppDatabaseV2
from obt.cpp_class_details import ClassDetailsDisplay
from obt.deco import Deco

deco = Deco()


class MembersCommand(CppCommandBase):
    """Command for displaying class/struct members"""
    
    def __init__(self):
        super().__init__(
            "obt.cpp.members",
            "Display members of C++ classes/structs"
        )
    
    def create_parser(self):
        parser = super().create_parser()
        
        # Add members-specific arguments
        parser.add_argument(
            'pattern',
            help='Class/struct name or pattern to show members for'
        )
        
        parser.add_argument(
            '--no-files',
            action='store_true',
            help='Hide file location information'
        )
        
        parser.add_argument(
            '--json',
            action='store_true',
            help='Output in JSON format (optimized for AI readability)'
        )
        
        # Filtering options
        parser.add_argument(
            '--access',
            nargs='+',
            choices=['public', 'protected', 'private'],
            help='Filter by access level(s)'
        )
        
        parser.add_argument(
            '--type',
            nargs='+',
            choices=['field', 'method', 'constructor', 'destructor', 'nested_type', 'typedef'],
            help='Filter by member type(s)'
        )
        
        parser.add_argument(
            '--static',
            action='store_true',
            help='Show only static members'
        )
        
        parser.add_argument(
            '--virtual',
            action='store_true',
            help='Show only virtual members'
        )
        
        parser.add_argument(
            '--const',
            action='store_true',
            help='Show only const members'
        )
        
        parser.add_argument(
            '--name-pattern',
            help='Filter by name regex pattern (e.g., "^get.*" for getters)'
        )
        
        return parser
    
    def run(self, args):
        """Execute the members command"""
        
        # Get or build database
        db = self.get_or_build_database(args)
        if not db:
            print(f"{deco.red('No database specified. Use --db or --source')}")
            sys.exit(1)
        
        try:
            # Create details display (which shows members)
            details_display = ClassDetailsDisplay(db)
            
            if args.json:
                # Build filters from command line arguments
                filters = {}
                
                if args.access:
                    filters['access_level'] = args.access
                
                if args.type:
                    filters['member_type'] = args.type
                
                if args.static:
                    filters['is_static'] = True
                
                if args.virtual:
                    filters['is_virtual'] = True
                
                if args.const:
                    filters['is_const'] = True
                
                if args.name_pattern:
                    filters['name_pattern'] = args.name_pattern
                
                # Output JSON
                json_output = details_display.to_json(args.pattern, filters if filters else None)
                print(json_output)
            else:
                # Display the details/members in colored format
                details_display.display_details(
                    args.pattern,
                    show_files=not args.no_files
                )
            
        except Exception as e:
            print(f"{deco.red(f'Error: {e}')}")
            sys.exit(1)
        finally:
            self.cleanup()


def main():
    """Main entry point"""
    cmd = MembersCommand()
    parser = cmd.create_parser()
    args = parser.parse_args()
    cmd.run(args)


if __name__ == '__main__':
    main()